@@ -22,6 +22,7 @@
 #include "debug.h"
 #include "sh2core.h"
 #include "bios.h"
+#include "smpc.h"
 
 static u8 sh2masklist[0x20] = {
 0xF0, 0xE0, 0xD0, 0xC0, 0xB0, 0xA0, 0x90, 0x80,
@@ -43,9 +44,6 @@ static u32 scumasklist[0x20] = {
 
 u32 interruptlist[2][0x80];
 
-void SmpcCKCHG320();
-void SmpcCKCHG352();
-
 //////////////////////////////////////////////////////////////////////////////
 
 void BiosInit(void)
@@ -125,8 +123,10 @@ void BiosInit(void)
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosSetScuInterrupt(SH2_struct * sh)
+static void FASTCALL BiosSetScuInterrupt(SH2_struct * sh)
 {
+   SH2GetRegisters(sh, &sh->regs);
+
 //   LOG("BiosSetScuInterrupt. vector = %02X, func = %08X\n", sh->regs.R[4], sh->regs.R[5]);
 
    if (sh->regs.R[5] == 0)
@@ -141,12 +141,15 @@ void FASTCALL BiosSetScuInterrupt(SH2_struct * sh)
    }
 
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosGetScuInterrupt(SH2_struct * sh)
+static void FASTCALL BiosGetScuInterrupt(SH2_struct * sh)
 {
+   SH2GetRegisters(sh, &sh->regs);
+
    // check me
 //   LOG("BiosGetScuInterrupt\n"); 
 
@@ -154,12 +157,15 @@ void FASTCALL BiosGetScuInterrupt(SH2_struct * sh)
    sh->cycles += 5;
 
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosSetSh2Interrupt(SH2_struct * sh)
+static void FASTCALL BiosSetSh2Interrupt(SH2_struct * sh)
 {
+   SH2GetRegisters(sh, &sh->regs);
+
 //   LOG("BiosSetSh2Interrupt\n");
 
    if (sh->regs.R[5] == 0)
@@ -174,12 +180,15 @@ void FASTCALL BiosSetSh2Interrupt(SH2_struct * sh)
    }
 
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosGetSh2Interrupt(SH2_struct * sh)
+static void FASTCALL BiosGetSh2Interrupt(SH2_struct * sh)
 {
+   SH2GetRegisters(sh, &sh->regs);
+
    // check me
 //   LOG("BiosGetSh2Interrupt\n");
 
@@ -187,12 +196,15 @@ void FASTCALL BiosGetSh2Interrupt(SH2_struct * sh)
    sh->cycles += 5;
 
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosSetScuInterruptMask(SH2_struct * sh)
+static void FASTCALL BiosSetScuInterruptMask(SH2_struct * sh)
 {
+   SH2GetRegisters(sh, &sh->regs);
+
    // check me
    LOG("BiosSetScuInterruptMask\n");
 
@@ -205,14 +217,17 @@ void FASTCALL BiosSetScuInterruptMask(SH2_struct * sh)
    sh->cycles += 17;
 
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosChangeScuInterruptMask(SH2_struct * sh)
+static void FASTCALL BiosChangeScuInterruptMask(SH2_struct * sh)
 {
    u32 newmask;
 
+   SH2GetRegisters(sh, &sh->regs);
+
 //   LOG("BiosChangeScuInterruptMask\n");
 
    // Read Stored Scu Interrupt Mask, AND it by R4, OR it by R5, then put it back
@@ -227,28 +242,35 @@ void FASTCALL BiosChangeScuInterruptMask(SH2_struct * sh)
    sh->cycles += 20;
 
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosCDINIT2(SH2_struct * sh)
+static void FASTCALL BiosCDINIT2(SH2_struct * sh)
 {
+   SH2GetRegisters(sh, &sh->regs);
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosCDINIT1(SH2_struct * sh)
+static void FASTCALL BiosCDINIT1(SH2_struct * sh)
 {
+   SH2GetRegisters(sh, &sh->regs);
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosGetSemaphore(SH2_struct * sh)
+static void FASTCALL BiosGetSemaphore(SH2_struct * sh)
 {
    u8 temp;
 
+   SH2GetRegisters(sh, &sh->regs);
+
    // check me
    LOG("BiosGetSemaphore\n");
   
@@ -262,12 +284,15 @@ void FASTCALL BiosGetSemaphore(SH2_struct * sh)
    
    sh->cycles += 11;
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosClearSemaphore(SH2_struct * sh)
+static void FASTCALL BiosClearSemaphore(SH2_struct * sh)
 {
+   SH2GetRegisters(sh, &sh->regs);
+
    // check me
    LOG("BiosClearSemaphore\n");
 
@@ -275,12 +300,15 @@ void FASTCALL BiosClearSemaphore(SH2_struct * sh)
 
    sh->cycles += 5;
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosChangeSystemClock(SH2_struct * sh)
+static void FASTCALL BiosChangeSystemClock(SH2_struct * sh)
 {
+   SH2GetRegisters(sh, &sh->regs);
+
    LOG("BiosChangeSystemClock\n");
 
    MappedMemoryWriteLong(0x06000324, sh->regs.R[4]);
@@ -291,14 +319,17 @@ void FASTCALL BiosChangeSystemClock(SH2_struct * sh)
       SmpcCKCHG352();
   
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosChangeScuInterruptPriority(SH2_struct * sh)
+static void FASTCALL BiosChangeScuInterruptPriority(SH2_struct * sh)
 {
    int i;
 
+   SH2GetRegisters(sh, &sh->regs);
+
    // check me
 //   LOG("BiosChangeScuInterruptPriority\n");
 
@@ -314,38 +345,48 @@ void FASTCALL BiosChangeScuInterruptPriority(SH2_struct * sh)
 
    sh->cycles += 186;
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosExecuteCDPlayer(SH2_struct * sh)
+static void FASTCALL BiosExecuteCDPlayer(SH2_struct * sh)
 {
+   SH2GetRegisters(sh, &sh->regs);
+
    LOG("BiosExecuteCDPlayer\n");
 
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosPowerOnMemoryClear(SH2_struct * sh)
+static void FASTCALL BiosPowerOnMemoryClear(SH2_struct * sh)
 {
+   SH2GetRegisters(sh, &sh->regs);
+
    LOG("BiosPowerOnMemoryClear\n");
 
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosCheckMPEGCard(SH2_struct * sh)
+static void FASTCALL BiosCheckMPEGCard(SH2_struct * sh)
 {
+   SH2GetRegisters(sh, &sh->regs);
+
    LOG("BiosCheckMPEGCard\n");
 
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-u32 GetDeviceStats(u32 device, u32 *size, u32 *addr, u32 *blocksize)
+static u32 GetDeviceStats(u32 device, u32 *size, u32 *addr, u32 *blocksize)
 {
    switch(device)
    {
@@ -380,14 +421,14 @@ u32 GetDeviceStats(u32 device, u32 *size, u32 *addr, u32 *blocksize)
 
 //////////////////////////////////////////////////////////////////////////////
 
-int CheckHeader(UNUSED u32 device)
+static int CheckHeader(UNUSED u32 device)
 {
    return 0;
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-int CalcSaveSize(u32 tableaddr, int blocksize)
+static int CalcSaveSize(u32 tableaddr, int blocksize)
 {
    int numblocks=0;
 
@@ -409,7 +450,7 @@ int CalcSaveSize(u32 tableaddr, int blocksize)
 
 //////////////////////////////////////////////////////////////////////////////
 
-u32 GetFreeSpace(UNUSED u32 device, u32 size, u32 addr, u32 blocksize)
+static u32 GetFreeSpace(UNUSED u32 device, u32 size, u32 addr, u32 blocksize)
 {
    u32 i;
    u32 usedblocks=0;
@@ -429,7 +470,7 @@ u32 GetFreeSpace(UNUSED u32 device, u32 size, u32 addr, u32 blocksize)
 
 //////////////////////////////////////////////////////////////////////////////
 
-u32 FindSave(UNUSED u32 device, u32 stringaddr, u32 blockoffset, u32 size, u32 addr, u32 blocksize)
+static u32 FindSave(UNUSED u32 device, u32 stringaddr, u32 blockoffset, u32 size, u32 addr, u32 blocksize)
 {
    u32 i;
 
@@ -470,7 +511,7 @@ u32 FindSave(UNUSED u32 device, u32 stringaddr, u32 blockoffset, u32 size, u32 a
 
 //////////////////////////////////////////////////////////////////////////////
 
-u32 FindSave2(UNUSED u32 device, const char *string, u32 blockoffset, u32 size, u32 addr, u32 blocksize)
+static u32 FindSave2(UNUSED u32 device, const char *string, u32 blockoffset, u32 size, u32 addr, u32 blocksize)
 {
    u32 i;
 
@@ -509,14 +550,14 @@ u32 FindSave2(UNUSED u32 device, const char *string, u32 blockoffset, u32 size,
 
 //////////////////////////////////////////////////////////////////////////////
 
-void DeleteSave(u32 addr, u32 blockoffset, u32 blocksize)
+static void DeleteSave(u32 addr, u32 blockoffset, u32 blocksize)
 {
     MappedMemoryWriteByte(addr + (blockoffset * blocksize * 2) + 0x1, 0x00);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-u16 *GetFreeBlocks(u32 addr, u32 blocksize, u32 numblocks, u32 size)
+static u16 *GetFreeBlocks(u32 addr, u32 blocksize, u32 numblocks, u32 size)
 {
    u8 *blocktbl;
    u16 *freetbl;
@@ -581,7 +622,7 @@ u16 *GetFreeBlocks(u32 addr, u32 blocksize, u32 numblocks, u32 size)
 
 //////////////////////////////////////////////////////////////////////////////
 
-u16 *ReadBlockTable(u32 addr, u32 *tableaddr, int block, int blocksize, int *numblocks, int *blocksread)
+static u16 *ReadBlockTable(u32 addr, u32 *tableaddr, int block, int blocksize, int *numblocks, int *blocksread)
 {
    u16 *blocktbl;
    int i=0;
@@ -618,8 +659,10 @@ u16 *ReadBlockTable(u32 addr, u32 *tableaddr, int block, int blocksize, int *num
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosBUPInit(SH2_struct * sh)
+static void FASTCALL BiosBUPInit(SH2_struct * sh)
 {
+   SH2GetRegisters(sh, &sh->regs);
+
 //   LOG("BiosBUPInit. arg1 = %08X, arg2 = %08X, arg3 = %08X\n", sh->regs.R[4], sh->regs.R[5], sh->regs.R[6]);
 
    // Setup Function table
@@ -662,33 +705,40 @@ void FASTCALL BiosBUPInit(SH2_struct * sh)
    // cycles need to be incremented
 
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosBUPSelectPartition(SH2_struct * sh)
+static void FASTCALL BiosBUPSelectPartition(SH2_struct * sh)
 {
+   SH2GetRegisters(sh, &sh->regs);
+
    LOG("BiosBUPSelectPartition. PR = %08X\n", sh->regs.PR);
 
    sh->regs.R[0] = 0; // returns 0 if there's no error
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosBUPFormat(SH2_struct * sh)
+static void FASTCALL BiosBUPFormat(SH2_struct * sh)
 {
+   SH2GetRegisters(sh, &sh->regs);
+
 //   LOG("BiosBUPFormat. PR = %08X\n", sh->regs.PR);
 
    BupFormat(sh->regs.R[4]);
 
    sh->regs.R[0] = 0; // returns 0 if there's no error
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosBUPStatus(SH2_struct * sh)
+static void FASTCALL BiosBUPStatus(SH2_struct * sh)
 {
    u32 size;
    u32 addr;
@@ -696,6 +746,8 @@ void FASTCALL BiosBUPStatus(SH2_struct * sh)
    u32 ret;
    u32 freeblocks=0;
 
+   SH2GetRegisters(sh, &sh->regs);
+
 //   LOG("BiosBUPStatus. arg1 = %d, arg2 = %d, arg3 = %08X, PR = %08X\n", sh->regs.R[4], sh->regs.R[5], sh->regs.R[6], sh->regs.PR);
 
    // Fill in status variables
@@ -707,6 +759,7 @@ void FASTCALL BiosBUPStatus(SH2_struct * sh)
       // Error
       sh->regs.R[0] = ret;
       sh->regs.PC = sh->regs.PR;
+      SH2SetRegisters(sh, &sh->regs);
       return;
    }
 
@@ -723,11 +776,12 @@ void FASTCALL BiosBUPStatus(SH2_struct * sh)
 
    sh->regs.R[0] = ret; // returns 0 if there's no error
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosBUPWrite(SH2_struct * sh)
+static void FASTCALL BiosBUPWrite(SH2_struct * sh)
 {
    u32 size;
    u32 addr;
@@ -741,6 +795,8 @@ void FASTCALL BiosBUPWrite(SH2_struct * sh)
    u32 datasize;
    u32 i;
 
+   SH2GetRegisters(sh, &sh->regs);
+
    LOG("BiosBUPWrite. arg1 = %d, arg2 = %08X, arg3 = %08X, arg4 = %d, PR = %08X\n", sh->regs.R[4], sh->regs.R[5], sh->regs.R[6], sh->regs.R[7], sh->regs.PR);
 
    // Fill in status variables
@@ -750,6 +806,7 @@ void FASTCALL BiosBUPWrite(SH2_struct * sh)
       // Error
       sh->regs.R[0] = ret;
       sh->regs.PC = sh->regs.PR;
+      SH2SetRegisters(sh, &sh->regs);
       return;
    }
 
@@ -764,6 +821,7 @@ void FASTCALL BiosBUPWrite(SH2_struct * sh)
          // Nope, let's bail instead
          sh->regs.R[0] = 6;
          sh->regs.PC = sh->regs.PR;
+         SH2SetRegisters(sh, &sh->regs);
          return;
       }
 
@@ -783,6 +841,7 @@ void FASTCALL BiosBUPWrite(SH2_struct * sh)
       // Nope, time to bail
       sh->regs.R[0] = 4;
       sh->regs.PC = sh->regs.PR;
+      SH2SetRegisters(sh, &sh->regs);
       return;
    }
 
@@ -792,6 +851,7 @@ void FASTCALL BiosBUPWrite(SH2_struct * sh)
       // Just return an error that might make sense
       sh->regs.R[0] = 8;
       sh->regs.PC = sh->regs.PR;
+      SH2SetRegisters(sh, &sh->regs);
       return;
    }
 
@@ -880,11 +940,12 @@ void FASTCALL BiosBUPWrite(SH2_struct * sh)
 
    sh->regs.R[0] = 0; // returns 0 if there's no error
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosBUPRead(SH2_struct * sh)
+static void FASTCALL BiosBUPRead(SH2_struct * sh)
 {
    u32 size;
    u32 addr;
@@ -897,6 +958,8 @@ void FASTCALL BiosBUPRead(SH2_struct * sh)
    int blocksread;
    u32 datasize;
 
+   SH2GetRegisters(sh, &sh->regs);
+
    LOG("BiosBUPRead\n", sh->regs.PR);
 
    ret = GetDeviceStats(sh->regs.R[4], &size, &addr, &blocksize);
@@ -906,6 +969,7 @@ void FASTCALL BiosBUPRead(SH2_struct * sh)
       // Error
       sh->regs.R[0] = ret;
       sh->regs.PC = sh->regs.PR;
+      SH2SetRegisters(sh, &sh->regs);
       return;
    }
 
@@ -915,6 +979,7 @@ void FASTCALL BiosBUPRead(SH2_struct * sh)
       // save doesn't exist
       sh->regs.R[0] = 5;
       sh->regs.PC = sh->regs.PR;
+      SH2SetRegisters(sh, &sh->regs);
       return;
    }
 
@@ -928,6 +993,7 @@ void FASTCALL BiosBUPRead(SH2_struct * sh)
       // Just return an error that might make sense
       sh->regs.R[0] = 8;
       sh->regs.PC = sh->regs.PR;
+      SH2SetRegisters(sh, &sh->regs);
       return;
    }
 
@@ -951,11 +1017,12 @@ void FASTCALL BiosBUPRead(SH2_struct * sh)
 
    sh->regs.R[0] = 0; // returns 0 if there's no error
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosBUPDelete(SH2_struct * sh)
+static void FASTCALL BiosBUPDelete(SH2_struct * sh)
 {
    u32 size;
    u32 addr;
@@ -963,6 +1030,8 @@ void FASTCALL BiosBUPDelete(SH2_struct * sh)
    u32 block;
    u32 ret;
 
+   SH2GetRegisters(sh, &sh->regs);
+
    LOG("BiosBUPDelete. PR = %08X\n", sh->regs.PR);
 
    // Fill in status variables
@@ -972,6 +1041,7 @@ void FASTCALL BiosBUPDelete(SH2_struct * sh)
       // Error
       sh->regs.R[0] = ret;
       sh->regs.PC = sh->regs.PR;
+      SH2SetRegisters(sh, &sh->regs);
       return;
    }
 
@@ -982,6 +1052,7 @@ void FASTCALL BiosBUPDelete(SH2_struct * sh)
 
       sh->regs.R[0] = 5;
       sh->regs.PC = sh->regs.PR;
+      SH2SetRegisters(sh, &sh->regs);
       return;
    }
 
@@ -989,11 +1060,12 @@ void FASTCALL BiosBUPDelete(SH2_struct * sh)
 
    sh->regs.R[0] = 0; // returns 0 if there's no error
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosBUPDirectory(SH2_struct * sh)
+static void FASTCALL BiosBUPDirectory(SH2_struct * sh)
 {
    u32 size;
    u32 addr;
@@ -1003,6 +1075,8 @@ void FASTCALL BiosBUPDirectory(SH2_struct * sh)
    char filename[12];
    u32 blockoffset=2;
 
+   SH2GetRegisters(sh, &sh->regs);
+
 //   int findmatch = MappedMemoryReadByte(sh->regs.R[5]);
 
    for (i = 0; i < 12; i++)
@@ -1017,6 +1091,7 @@ void FASTCALL BiosBUPDirectory(SH2_struct * sh)
       // Error
       sh->regs.R[0] = ret;
       sh->regs.PC = sh->regs.PR;
+      SH2SetRegisters(sh, &sh->regs);
       return;
    }
 
@@ -1082,11 +1157,12 @@ void FASTCALL BiosBUPDirectory(SH2_struct * sh)
 
    sh->regs.R[0] = i; // returns the number of successfully read dir entries
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosBUPVerify(SH2_struct * sh)
+static void FASTCALL BiosBUPVerify(SH2_struct * sh)
 {
    u32 size;
    u32 addr;
@@ -1099,6 +1175,8 @@ void FASTCALL BiosBUPVerify(SH2_struct * sh)
    int numblocks;
    int blocksread;
 
+   SH2GetRegisters(sh, &sh->regs);
+
    LOG("BiosBUPVerify. PR = %08X\n", sh->regs.PR);
 
    ret = GetDeviceStats(sh->regs.R[4], &size, &addr, &blocksize);
@@ -1108,6 +1186,7 @@ void FASTCALL BiosBUPVerify(SH2_struct * sh)
       // Error
       sh->regs.R[0] = ret;
       sh->regs.PC = sh->regs.PR;
+      SH2SetRegisters(sh, &sh->regs);
       return;
    }
 
@@ -1117,6 +1196,7 @@ void FASTCALL BiosBUPVerify(SH2_struct * sh)
       // Since the save doesn't exist, let's bail with an error
       sh->regs.R[0] = 5; // Not found
       sh->regs.PC = sh->regs.PR;
+      SH2SetRegisters(sh, &sh->regs);
       return;
    }
 
@@ -1130,6 +1210,7 @@ void FASTCALL BiosBUPVerify(SH2_struct * sh)
       // Just return an error that might make sense
       sh->regs.R[0] = 8; // Broken
       sh->regs.PC = sh->regs.PR;
+      SH2SetRegisters(sh, &sh->regs);
       return;
    }
 
@@ -1142,6 +1223,7 @@ void FASTCALL BiosBUPVerify(SH2_struct * sh)
          // Ok, the data doesn't match
          sh->regs.R[0] = 7; // No match
          sh->regs.PC = sh->regs.PR;
+         SH2SetRegisters(sh, &sh->regs);
          return;
       }
 
@@ -1161,11 +1243,12 @@ void FASTCALL BiosBUPVerify(SH2_struct * sh)
 
    sh->regs.R[0] = 0; // returns 0 if there's no error
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void ConvertMonthAndDay(u32 data, u32 monthaddr, u32 dayaddr, int type)
+static void ConvertMonthAndDay(u32 data, u32 monthaddr, u32 dayaddr, int type)
 {
    int i;
    u16 monthtbl[11] = { 31, 31+28, 31+28+31, 31+28+31+30, 31+28+31+30+31,
@@ -1213,13 +1296,15 @@ void ConvertMonthAndDay(u32 data, u32 monthaddr, u32 dayaddr, int type)
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosBUPGetDate(SH2_struct * sh)
+static void FASTCALL BiosBUPGetDate(SH2_struct * sh)
 {
    u32 date;
    u32 div;
    u32 yearoffset;
    u32 yearremainder;
 
+   SH2GetRegisters(sh, &sh->regs);
+
    LOG("BiosBUPGetDate. PR = %08X\n", sh->regs.PR);
 
    date = sh->regs.R[4];
@@ -1255,11 +1340,12 @@ void FASTCALL BiosBUPGetDate(SH2_struct * sh)
    MappedMemoryWriteByte(sh->regs.R[5], (u8)(((div / 0x5B5) * 4) + yearoffset));
    
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosBUPSetDate(SH2_struct * sh)
+static void FASTCALL BiosBUPSetDate(SH2_struct * sh)
 {
    u32 date;
    u8 data;
@@ -1270,6 +1356,8 @@ void FASTCALL BiosBUPSetDate(SH2_struct * sh)
                         31+28+31+30+31+30+31+31+30+31,
                         31+28+31+30+31+30+31+31+30+31+30 };
 
+   SH2GetRegisters(sh, &sh->regs);
+
    LOG("BiosBUPSetDate. PR = %08X\n", sh->regs.PR);
 
    // Year
@@ -1300,12 +1388,15 @@ void FASTCALL BiosBUPSetDate(SH2_struct * sh)
 
    sh->regs.R[0] = date;
    sh->regs.PC = sh->regs.PR;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosHandleScuInterrupt(SH2_struct * sh, int vector)
+static void FASTCALL BiosHandleScuInterrupt(SH2_struct * sh, int vector)
 {
+   SH2GetRegisters(sh, &sh->regs);
+
    // Save R0-R7, PR, GBR, and old Interrupt mask to stack
    sh->regs.R[15] -= 4;
    MappedMemoryWriteLong(sh->regs.R[15], sh->regs.R[0]);
@@ -1345,14 +1436,17 @@ void FASTCALL BiosHandleScuInterrupt(SH2_struct * sh, int vector)
 //   LOG("Interrupt PC = %08X. Read from %08X\n", sh->regs.PC, 0x06000900+(vector << 2));
 
    sh->cycles += 33;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL BiosHandleScuInterruptReturn(SH2_struct * sh)
+static void FASTCALL BiosHandleScuInterruptReturn(SH2_struct * sh)
 {
    u32 oldmask;
 
+   SH2GetRegisters(sh, &sh->regs);
+
    // Restore R0-R7, PR, GBR, and old Interrupt mask from stack
    sh->regs.GBR = MappedMemoryReadLong(sh->regs.R[15]);
    sh->regs.R[15] += 4;
@@ -1387,12 +1481,15 @@ void FASTCALL BiosHandleScuInterruptReturn(SH2_struct * sh)
    sh->regs.R[15] += 4;
 
    sh->cycles += 24;
+   SH2SetRegisters(sh, &sh->regs);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
 int FASTCALL BiosHandleFunc(SH2_struct * sh)
 {
+   SH2GetRegisters(sh, &sh->regs);
+
    // Let's see if it's a bios function
    switch((sh->regs.PC - 0x200) >> 2)
    {
