@@ -56,7 +56,29 @@
 #endif 
 #endif
 
-#ifndef GEKKO
+#ifdef GEKKO
+/* Wii have both stdint.h and "yabause" definitions of fixed
+size types */
+#include <gccore.h>
+typedef unsigned long pointer;
+
+#else /* ! GEKKO */
+
+#ifdef HAVE_STDINT_H
+
+#include <stdint.h>
+typedef uint8_t u8;
+typedef  int8_t s8;
+typedef uint16_t u16;
+typedef  int16_t s16;
+typedef uint32_t u32;
+typedef  int32_t s32;
+typedef uint64_t u64;
+typedef  int64_t s64;
+typedef uintptr_t pointer;
+
+#else  // !HAVE_STDINT_H
+
 typedef unsigned char u8;
 typedef unsigned short u16;
 
@@ -95,11 +117,10 @@ typedef unsigned long pointer;
 typedef signed long s32;
 typedef signed long long s64;
 #endif
-#else
-// Gamecube/Wii
-#include <gccore.h>
-typedef unsigned long pointer;
-#endif
+
+#endif  // !HAVE_STDINT_H
+
+#endif // !GEKKO
 
 typedef struct {
 	unsigned int size;
@@ -182,6 +203,47 @@ static INLINE int StateCheckRetrieveHeader(FILE *fp, const char *name, int *vers
 
 //////////////////////////////////////////////////////////////////////////////
 
+/* Minimum/maximum values */
+
+#undef MIN
+#undef MAX
+#define MIN(a,b)  ((a) < (b) ? (a) : (b))
+#define MAX(a,b)  ((a) > (b) ? (a) : (b))
+
+//////////////////////////////////////////////////////////////////////////////
+
+/*
+ * BSWAP16(x) swaps two bytes in a 16-bit value (AABB -> BBAA) or adjacent
+ * bytes in a 32-bit value (AABBCCDD -> BBAADDCC).
+ *
+ * BSWAP32(x) reverses four bytes in a 32-bit value (AABBCCDD -> DDCCBBAA).
+ *
+ * WSWAP32(x) swaps two 16-bit words in a 32-bit value (AABBCCDD -> CCDDAABB).
+ *
+ * Any of these can be left undefined if there is no platform-specific
+ * optimization for them; the defaults below will then be used instead.
+ */
+
+#ifdef PSP
+# define BSWAP16(x)  ((typeof(x)) __builtin_allegrex_wsbh((x)))
+# define BSWAP32(x)  ((typeof(x)) __builtin_allegrex_wsbw((x)))
+# define WSWAP32(x)  ((typeof(x)) __builtin_allegrex_rotr((x), 16))
+#endif
+
+/* Defaults: */
+
+#ifndef BSWAP16
+# define BSWAP16(x)  (((u32)(x)>>8 & 0x00FF00FF) | ((u32)(x) & 0x00FF00FF) << 8)
+#endif
+#ifndef BSWAP32
+# define BSWAP32(x)  ((u32)(x)>>24 | ((u32)(x)>>8 & 0xFF00) | ((u32)(x) & 0xFF00)<<8 | (u32)(x)<<24)
+#endif
+#ifndef WSWAP32
+# define WSWAP32(x)  ((u32)(x)>>16 | (u32)(x)<<16)
+#endif
+
+//////////////////////////////////////////////////////////////////////////////
+
 #ifdef __GNUC__
 
 #define UNUSED __attribute ((unused))
