@@ -17,10 +17,10 @@
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
-#include "dreamcast/perdc.h"
-#include "yabause.h"
-#include "yui.h"
-#include "vdp2.h"
+#include "perdc.h"
+#include "../yabause.h"
+#include "../yui.h"
+#include "../vdp2.h"
 
 #include <dc/maple.h>
 #include <dc/maple/controller.h>
@@ -28,15 +28,21 @@
 int PERDCInit(void);
 void PERDCDeInit(void);
 int PERDCHandleEvents(void);
+void PERDCNothing(void);
+u32 PERDCScan(void);
 
 static PerPad_struct *pad1;
 
 PerInterface_struct PERDC = {
-	PERCORE_DC,
-	"Dreamcast Input Interface",
-	PERDCInit,
-	PERDCDeInit,
-	PERDCHandleEvents
+    PERCORE_DC,
+    "Dreamcast Input Interface",
+    PERDCInit,
+    PERDCDeInit,
+    PERDCHandleEvents,
+    PERDCNothing,
+    PERDCScan,
+    0,
+    PERDCNothing
 };
 
 int PERDCInit(void)	{
@@ -128,3 +134,12 @@ int PERDCHandleEvents(void)	{
 
 	return 0;
 }
+
+void PERDCNothing(void) {
+    /* Nothing */
+}
+
+u32 PERDCScan(void) {
+    /* Nothing */
+    return 0;
+}
