@@ -16,13 +16,14 @@
 
     You should have received a copy of the GNU General Public License
     along with Yabause; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
 #include "gtkglwidget.h"
 #include "yuiwindow.h"
 
 #include "../yabause.h"
+#include "../yui.h"
 #include "../peripheral.h"
 #include "../sh2core.h"
 #include "../sh2int.h"
@@ -33,11 +34,16 @@
 #include "../cdbase.h"
 #include "../scsp.h"
 #include "../sndsdl.h"
+#include "../sndal.h"
 #include "../persdljoy.h"
+#ifdef ARCH_IS_LINUX
+#include "../perlinuxjoy.h"
+#endif
 #include "../debug.h"
 #include "../m68kcore.h"
 #include "../m68kc68k.h"
 #include "pergtk.h"
+#include "../psp/psp-sh2.h"
 
 #include "settings.h"
 
@@ -52,12 +58,21 @@ M68K_struct * M68KCoreList[] = {
 #ifdef HAVE_C68K
 &M68KC68K,
 #endif
+#ifdef HAVE_Q68
+&M68KQ68,
+#endif
 NULL
 };
 
 SH2Interface_struct *SH2CoreList[] = {
 &SH2Interpreter,
 &SH2DebugInterpreter,
+#ifdef TEST_PSP_SH2
+&SH2PSP,
+#endif
+#ifdef SH2_DYNAREC
+&SH2Dynarec,
+#endif
 NULL
 };
 
@@ -67,6 +82,9 @@ PerInterface_struct *PERCoreList[] = {
 #ifdef HAVE_LIBSDL
 &PERSDLJoy,
 #endif
+#ifdef ARCH_IS_LINUX
+&PERLinuxJoy,
+#endif
 NULL
 };
 
@@ -84,6 +102,9 @@ SoundInterface_struct *SNDCoreList[] = {
 #ifdef HAVE_LIBSDL
 &SNDSDL,
 #endif
+#ifdef HAVE_LIBAL
+&SNDAL,
+#endif
 NULL
 };
 
@@ -100,16 +121,12 @@ GtkWidget * yui;
 GKeyFile * keyfile;
 yabauseinit_struct yinit;
 
-const char * key_names[] = { "Up", "Right", "Down", "Left", "Right trigger", "Left trigger",
-	"Start", "A", "B", "C", "X", "Y", "Z", NULL };
-const char * mouse_key_names[] = { "Left", "Middle", "Right", "Start", NULL };
-
-int yui_main(gpointer data) {
+static int yui_main(gpointer data) {
 	PERCore->HandleEvents();
 	return TRUE;
 }
 
-GtkWidget * yui_new() {
+static GtkWidget * yui_new(void) {
 	yui = yui_window_new(NULL, G_CALLBACK(YabauseInit), &yinit, yui_main, G_CALLBACK(YabauseReset));
 
 	gtk_widget_show(yui);
@@ -117,7 +134,7 @@ GtkWidget * yui_new() {
 	return yui;
 }
 
-void yui_settings_init(void) {
+static void yui_settings_init(void) {
 	yinit.m68kcoretype = M68KCORE_C68K;
 	yinit.percoretype = PERCORE_GTK;
 	yinit.sh2coretype = SH2CORE_DEFAULT;
@@ -135,12 +152,12 @@ void yui_settings_init(void) {
 	yinit.buppath = buppath;
 	yinit.mpegpath = mpegpath;
 	yinit.cartpath = cartpath;
-        yinit.flags = VIDEOFORMATTYPE_NTSC;
+        yinit.videoformattype = VIDEOFORMATTYPE_NTSC;
 }
 
 gchar * inifile;
 
-int safe_strcmp(const char * s1, const char * s2) {
+static int safe_strcmp(const char * s1, const char * s2) {
 	if (s1) {
 		if (s2) {
 			return strcmp(s1, s2);
@@ -159,8 +176,9 @@ int safe_strcmp(const char * s1, const char * s2) {
 extern void * padbits;
 void * padbits;
 
-gboolean yui_settings_load(void) {
+static gboolean yui_settings_load(void) {
 	int i, tmp;
+	long tmptime;
 	gchar * stmp;
 	gboolean mustRestart = FALSE;
 
@@ -194,8 +212,8 @@ gboolean yui_settings_load(void) {
 
 	/* region */
 	{
-		tmp = yinit.regionid;
 		char * region = g_key_file_get_value(keyfile, "General", "Region", 0);
+		tmp = yinit.regionid;
 		if ((region == 0) || !strcmp(region, "Auto")) {
 			yinit.regionid = 0;
 		} else {
@@ -265,6 +283,13 @@ gboolean yui_settings_load(void) {
 		mustRestart = TRUE;
 	}
 
+	/* m68k */
+	tmp = yinit.m68kcoretype;
+	yinit.m68kcoretype = g_key_file_get_integer(keyfile, "General", "M68kInt", 0);
+	if ((YUI_WINDOW(yui)->state & YUI_IS_INIT) && (tmp != yinit.m68kcoretype)) {
+		mustRestart = TRUE;
+	}
+
 	/* video core */
 	tmp = yinit.vidcoretype;
 	yinit.vidcoretype = g_key_file_get_integer(keyfile, "General", "VideoCore", 0);
@@ -286,6 +311,45 @@ gboolean yui_settings_load(void) {
 	/* peripheral core */
 	yinit.percoretype = g_key_file_get_integer(keyfile, "General", "PerCore", 0);
 
+	/* audio sync */
+	tmp = g_key_file_get_boolean(keyfile, "General", "AudioSync", 0);
+	ScspSetFrameAccurate(tmp);
+
+	/* clock sync */
+	tmp = yinit.clocksync;
+	yinit.clocksync = g_key_file_get_boolean(keyfile, "General", "ClockSync", 0);
+	if ((YUI_WINDOW(yui)->state & YUI_IS_INIT) && (tmp != yinit.clocksync)) {
+		mustRestart = TRUE;
+	}
+	tmptime = yinit.basetime;
+	tmp = g_key_file_get_boolean(keyfile, "General", "FixedBaseTime", 0);
+	if (tmp && yinit.clocksync) {
+		/* Find timestamp of 1998-01-01 12:00 in the local time zone */
+		time_t utc = 883656000;  // 1998-01-01 12:00 UTC
+		struct tm tm;
+		long local, noon;
+		localtime_r(&utc, &tm);
+		local = tm.tm_hour*3600 + tm.tm_min*60 + tm.tm_sec;
+		if (tm.tm_mday == 2)  // 1998-01-02
+			local += 86400;
+		else if (tm.tm_mday == 31)  // 1997-12-31
+			local -= 86400;
+		noon = 12*3600 + 0*60 + 0;
+		yinit.basetime = (long)utc + (noon - local);
+	} else {
+		yinit.basetime = 0;
+	}
+	if ((YUI_WINDOW(yui)->state & YUI_IS_INIT) && (tmptime != yinit.basetime)) {
+		mustRestart = TRUE;
+	}
+
+	/* threads */
+	tmp = g_key_file_get_boolean(keyfile, "General", "UseThreads", 0);
+	if ((YUI_WINDOW(yui)->state & YUI_IS_INIT) && (tmp != yinit.usethreads)) {
+		mustRestart = TRUE;
+	}
+	yinit.usethreads = tmp;
+
 	PerInit(yinit.percoretype);
 
 	PerPortReset();
@@ -295,10 +359,11 @@ gboolean yui_settings_load(void) {
 			padbits = PerMouseAdd(&PORTDATA1);
 			i = 0;
 
-			while(mouse_key_names[i]) {
+			while(PerMouseNames[i]) {
 				char tmp[100];
-				sprintf(tmp, "Mouse.%s.1", mouse_key_names[i]);
-	  			u32 key = g_key_file_get_integer(keyfile, PERCore->Name, tmp, 0);
+				u32 key;
+				sprintf(tmp, "Mouse.%s.1", PerMouseNames[i]);
+	  			key = g_key_file_get_integer(keyfile, PERCore->Name, tmp, 0);
 		  		PerSetKey(key, i + 13, padbits);
 		  		i++;
 			}
@@ -308,10 +373,11 @@ gboolean yui_settings_load(void) {
 			padbits = PerPadAdd(&PORTDATA1);
 			i = 0;
 
-			while(key_names[i]) {
+			while(PerPadNames[i]) {
 				char tmp[100];
-				sprintf(tmp, "Pad.%s.1", key_names[i]);
-	  			u32 key = g_key_file_get_integer(keyfile, PERCore->Name, tmp, 0);
+				u32 key;
+				sprintf(tmp, "Pad.%s.1", PerPadNames[i]);
+	  			key = g_key_file_get_integer(keyfile, PERCore->Name, tmp, 0);
 		  		PerSetKey(key, i, padbits);
 		  		i++;
 			}
@@ -321,7 +387,9 @@ gboolean yui_settings_load(void) {
 			g_key_file_get_integer(keyfile, "General", "Height", 0),
 			g_key_file_get_integer(keyfile, "General", "Fullscreen", 0));
 
-        yinit.flags = g_key_file_get_integer(keyfile, "General", "VideoFormat", 0);
+        yinit.videoformattype = g_key_file_get_integer(keyfile, "General", "VideoFormat", 0);
+
+	yui_window_set_frameskip(YUI_WINDOW(yui), g_key_file_get_integer(keyfile, "General", "Frameskip", NULL));
 
 	return mustRestart;
 }
@@ -366,6 +434,7 @@ int main(int argc, char *argv[]) {
 	yui_settings_init();
 
 #ifdef HAVE_LIBMINI18N
+	mini18n_set_domain(YTSDIR);
 	g_key_file_load_from_file(keyfile, inifile, G_KEY_FILE_NONE, 0);
 	mini18n_set_locale(g_key_file_get_value(keyfile, "General", "TranslationPath", NULL));
 #endif
@@ -414,25 +483,39 @@ int main(int argc, char *argv[]) {
          else if (strcmp(argv[i], "-ns") == 0 || strcmp(argv[i], "--nosound") == 0) {
 	    yinit.sndcoretype = 0;
 	 }
+	 // Autoload
+	 else if (strcmp(argv[i], "--autoload") == 0) {
+            yui_window_start(YUI_WINDOW(yui));
+            YuiLoadState();
+            yui_window_run(YUI_WINDOW(yui));
+	 }
 	 // Autostart
 	 else if (strcmp(argv[i], "-a") == 0 || strcmp(argv[i], "--autostart") == 0) {
-            yui_window_run(NULL, YUI_WINDOW(yui));
+            yui_window_run(YUI_WINDOW(yui));
 	 }
 	 // Fullscreen
 	 else if (strcmp(argv[i], "-f") == 0 || strcmp(argv[i], "--fullscreen") == 0) {
             yui_window_set_fullscreen(YUI_WINDOW(yui), TRUE);
 	 }
+         // Auto frame skip
+         else if (strstr(argv[i], "--autoframeskip=")) {
+            int fscount;
+            int fsenable;
+            fscount = sscanf(argv[i] + strlen("--autoframeskip="), "%d", &fsenable);
+            if (fscount > 0)
+               yui_window_set_frameskip(YUI_WINDOW(yui), fsenable);
+         }
 	 // Binary
 	 else if (strstr(argv[i], "--binary=")) {
 	    char binname[1024];
-	    u32 binaddress;
+	    unsigned int binaddress;
 	    int bincount;
 
 	    bincount = sscanf(argv[i] + strlen("--binary="), "%[^:]:%x", binname, &binaddress);
 	    if (bincount > 0) {
 	       if (bincount < 2) binaddress = 0x06004000;
 
-               yui_window_run(NULL, YUI_WINDOW(yui));
+               yui_window_run(YUI_WINDOW(yui));
 	       MappedMemoryLoadExec(binname, binaddress);
 	    }
 	 }
@@ -448,9 +531,6 @@ int main(int argc, char *argv[]) {
 	return 0;
 }
 
-void YuiVideoResize(unsigned int w, unsigned int h, int isfullscreen) {
-}
-
 void YuiSetVideoAttribute(int type, int val) {
 }
 
