@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with Yabause; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
 #include "settings.h"
@@ -30,15 +30,7 @@
 #include "yuimem.h"
 #include "yuiscreenshot.h"
 
-void ToggleFLimiter (GtkWidget* widget, gpointer user_data)
-{
-	if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
-		EnableAutoFrameSkip ();
-	else
-		DisableAutoFrameSkip ();
-}
-
-void openAboutDialog(GtkWidget * w, gpointer data) {
+static void openAboutDialog(GtkWidget * w, gpointer data) {
 	gtk_show_about_dialog(data,
 		"name", "Yabause",
 		"version", VERSION,
@@ -46,13 +38,13 @@ void openAboutDialog(GtkWidget * w, gpointer data) {
 		NULL);
 }
 
-void YuiSaveState() {
+void YuiSaveState(void) {
   char * dir = g_key_file_get_value(keyfile, "General", "StatePath", NULL);
 
   YabSaveStateSlot(dir, 1);
 }
 
-void YuiLoadState() {
+void YuiLoadState(void) {
   char * dir = g_key_file_get_value(keyfile, "General", "StatePath", NULL);
 
   YabLoadStateSlot(dir, 1);
@@ -120,7 +112,10 @@ GtkWidget* create_menu(YuiWindow * window1) {
   g_signal_connect_swapped(screenshot, "activate", G_CALLBACK(yui_screenshot_new), window1);
 
   frameLimiter = gtk_check_menu_item_new_with_mnemonic (_("Frame Skip/Limiter"));
-  g_signal_connect (frameLimiter, "activate", G_CALLBACK (ToggleFLimiter), NULL);
+  {
+    GtkAction * action = gtk_action_group_get_action(window1->action_group, "frameskip");
+    gtk_action_connect_proxy(action, frameLimiter);
+  }
   gtk_container_add (GTK_CONTAINER (menuitem1_menu), frameLimiter);
 
   {
@@ -164,35 +159,12 @@ GtkWidget* create_menu(YuiWindow * window1) {
   layer1_menu = gtk_menu_new ();
   gtk_menu_item_set_submenu (GTK_MENU_ITEM (layer1), layer1_menu);
 
-  vdp3 = gtk_check_menu_item_new_with_mnemonic ("Vdp1");
-  gtk_container_add (GTK_CONTAINER (layer1_menu), vdp3);
-  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (vdp3), TRUE);
-  g_signal_connect(vdp3, "activate", G_CALLBACK(ToggleVDP1), 0);
-
-  nbg1 = gtk_check_menu_item_new_with_mnemonic ("NBG0");
-  gtk_container_add (GTK_CONTAINER (layer1_menu), nbg1);
-  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (nbg1), TRUE);
-  g_signal_connect(nbg1, "activate", G_CALLBACK(ToggleNBG0), 0);
-
-  nbg2 = gtk_check_menu_item_new_with_mnemonic ("NBG1");
-  gtk_container_add (GTK_CONTAINER (layer1_menu), nbg2);
-  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (nbg2), TRUE);
-  g_signal_connect(nbg2, "activate", G_CALLBACK(ToggleNBG1), 0);
-
-  nbg3 = gtk_check_menu_item_new_with_mnemonic ("NBG2");
-  gtk_container_add (GTK_CONTAINER (layer1_menu), nbg3);
-  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (nbg3), TRUE);
-  g_signal_connect(nbg3, "activate", G_CALLBACK(ToggleNBG2), 0);
-
-  nbg4 = gtk_check_menu_item_new_with_mnemonic ("NBG3");
-  gtk_container_add (GTK_CONTAINER (layer1_menu), nbg4);
-  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (nbg4), TRUE);
-  g_signal_connect(nbg4, "activate", G_CALLBACK(ToggleNBG3), 0);
-
-  rbg1 = gtk_check_menu_item_new_with_mnemonic ("RBG1");
-  gtk_container_add (GTK_CONTAINER (layer1_menu), rbg1);
-  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (rbg1), TRUE);
-  g_signal_connect(rbg1, "activate", G_CALLBACK(ToggleRBG0), 0);
+  gtk_container_add(GTK_CONTAINER(layer1_menu), gtk_action_create_menu_item(gtk_action_group_get_action(window1->action_group, "toggle_vdp1")));
+  gtk_container_add(GTK_CONTAINER(layer1_menu), gtk_action_create_menu_item(gtk_action_group_get_action(window1->action_group, "toggle_nbg0")));
+  gtk_container_add(GTK_CONTAINER(layer1_menu), gtk_action_create_menu_item(gtk_action_group_get_action(window1->action_group, "toggle_nbg1")));
+  gtk_container_add(GTK_CONTAINER(layer1_menu), gtk_action_create_menu_item(gtk_action_group_get_action(window1->action_group, "toggle_nbg2")));
+  gtk_container_add(GTK_CONTAINER(layer1_menu), gtk_action_create_menu_item(gtk_action_group_get_action(window1->action_group, "toggle_nbg3")));
+  gtk_container_add(GTK_CONTAINER(layer1_menu), gtk_action_create_menu_item(gtk_action_group_get_action(window1->action_group, "toggle_rbg0")));
 
   gtk_container_add(GTK_CONTAINER(view1_menu), gtk_action_create_menu_item(gtk_action_group_get_action(window1->action_group, "fullscreen")));
 
