@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with Yabause; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
 #include "pergtk.h"
@@ -29,7 +29,7 @@ void PERGTKDeInit(void);
 int PERGTKHandleEvents(void);
 void PERGTKNothing(void);
 
-u32 PERGTKScan(const char * name);
+u32 PERGTKScan(void);
 void PERGTKFlush(void);
 void PERGTKKeyName(u32 key, char * name, int size);
 
@@ -74,7 +74,7 @@ int PERGTKHandleEvents(void) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-u32 PERGTKScan(UNUSED const char * name) {
+u32 PERGTKScan(void) {
 	g_print("this is wrong, the gtk peripheral can't scan\n");
 	return 1;
 }
