@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with Yabause; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
 #include "yuiviewer.h"
@@ -108,8 +108,8 @@ static void yui_vdp1_init (YuiVdp1 * yv) {
 	gtk_paned_pack2(GTK_PANED(hbox), vbox2, TRUE, TRUE);
 	{
 		GtkWidget * scroll = gtk_scrolled_window_new(NULL, NULL);
-		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 		GtkWidget * text = gtk_text_view_new();
+		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 		gtk_text_view_set_editable(GTK_TEXT_VIEW(text), FALSE);
 		gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(text), FALSE);
 		yv->buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text));
@@ -144,8 +144,8 @@ GtkWidget * yui_vdp1_new(YuiWindow * y) {
 	yv->yui = y;
 
 	if (!( yv->yui->state & YUI_IS_INIT )) {
-	  yui_window_run(dialog, yv->yui);
-	  yui_window_pause(dialog, yv->yui);
+	  yui_window_run(yv->yui);
+	  yui_window_pause(yv->yui);
 	}
 
 	{
