@@ -15,12 +15,39 @@
 
     You should have received a copy of the GNU General Public License
     along with Yabause; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
 #include "debug.h"
 #include "peripheral.h"
 
+const char * PerPadNames[] =
+{
+"Up",
+"Right",
+"Down",
+"Left",
+"R",
+"L",
+"Start",
+"A",
+"B",
+"C",
+"X",
+"Y",
+"Z",
+NULL
+};
+
+const char * PerMouseNames[] =
+{
+"A",
+"B",
+"C",
+"Start",
+NULL
+};
+
 PortData_struct PORTDATA1;
 PortData_struct PORTDATA2;
 
@@ -478,6 +505,7 @@ void * PerAddPeripheral(PortData_struct *port, int perid)
       for(i = 0;i < (pernum - current);i++)
       {
          port->data[tmp + i] = 0xFF;
+         port->size++;
       }
    }
 
@@ -496,6 +524,14 @@ void * PerAddPeripheral(PortData_struct *port, int perid)
 
 //////////////////////////////////////////////////////////////////////////////
 
+int PerGetId(void * peripheral)
+{
+   u8 * id = peripheral;
+   return *id;
+}
+
+//////////////////////////////////////////////////////////////////////////////
+
 void PerRemovePeripheral(UNUSED PortData_struct *port, UNUSED int removeoffset)
 {
    // stub
@@ -625,7 +661,7 @@ void PERDummyNothing(void);
 //static PortData_struct port1;
 //static PortData_struct port2;
 
-u32 PERDummyScan(const char *);
+u32 PERDummyScan(void);
 void PERDummyFlush(void);
 void PERDummyKeyName(u32 key, char * name, int size);
 
@@ -672,7 +708,7 @@ int PERDummyHandleEvents(void) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-u32 PERDummyScan(UNUSED const char * name) {
+u32 PERDummyScan(void) {
    return 0;
 }
 
