@@ -16,7 +16,7 @@
 
 	You should have received a copy of the GNU General Public License
 	along with Yabause; if not, write to the Free Software
-	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 #include "PerQt.h"
 
@@ -25,7 +25,7 @@ void PERQTDeInit(void);
 int PERQTHandleEvents(void);
 void PERQTNothing(void);
 
-u32 PERQTScan(const char* name);
+u32 PERQTScan(void);
 void PERQTFlush(void);
 
 PerInterface_struct PERQT = {
@@ -56,8 +56,8 @@ int PERQTHandleEvents(void)
 	return 0;
 }
 
-u32 PERQTScan(const char* name)
-{ return 1; }
+u32 PERQTScan(void)
+{ return 0; }
 
 void PERQTFlush(void)
 {}
