@@ -21,11 +21,13 @@
 #include "QtYabause.h"
 #include "ui/UIYabause.h"
 #include "Settings.h"
+#include "VolatileSettings.h"
 
 #include <QApplication>
 #include <QLabel>
 #include <QGroupBox>
 #include <QTreeWidget>
+#include <QPointer>
 
 // cores
 
@@ -33,6 +35,7 @@
 extern CDInterface SPTICD;
 #endif
 
+extern "C" {
 M68K_struct * M68KCoreList[] = {
 &M68KDummy,
 #ifdef HAVE_C68K
@@ -44,6 +47,9 @@ NULL
 SH2Interface_struct *SH2CoreList[] = {
 &SH2Interpreter,
 &SH2DebugInterpreter,
+#ifdef SH2_DYNAREC
+&SH2Dynarec,
+#endif
 NULL
 };
 
@@ -62,7 +68,9 @@ NULL
 CDInterface *CDCoreList[] = {
 &DummyCD,
 &ISOCD,
+#ifndef UNKNOWN_ARCH
 &ArchCD,
+#endif
 NULL
 };
 
@@ -86,10 +94,13 @@ VideoInterface_struct *VIDCoreList[] = {
 NULL
 };
 
+}
+
 // main window
-UIYabause* mUIYabause = 0;
+QPointer<UIYabause> mUIYabause = 0;
 // settings object
-Settings* mSettings = 0;
+QPointer<Settings> mSettings = 0;
+QPointer<VolatileSettings> mVolatileSettings = 0;
 // ports padbits
 QMap<uint, PerPad_struct*> mPort1PadsBits;
 QMap<uint, PerPad_struct*> mPort2PadsBits;
@@ -109,20 +120,27 @@ extern "C"
 	{ QtYabause::mainWindow()->swapBuffers(); }
 }
 
-UIYabause* QtYabause::mainWindow()
+UIYabause* QtYabause::mainWindow( bool create )
 {
-	if ( !mUIYabause )
+	if ( !mUIYabause && create )
 		mUIYabause = new UIYabause;
 	return mUIYabause;
 }
 
-Settings* QtYabause::settings()
+Settings* QtYabause::settings( bool create )
 {
-	if ( !mSettings )
+	if ( !mSettings && create )
 		mSettings = new Settings();
 	return mSettings;
 }
 
+VolatileSettings* QtYabause::volatileSettings( bool create )
+{
+	if ( !mVolatileSettings && create )
+		mVolatileSettings = new VolatileSettings();
+	return mVolatileSettings;
+}
+
 int QtYabause::setTranslationFile()
 {
 #ifdef HAVE_LIBMINI18N
@@ -318,7 +336,11 @@ VideoInterface_struct* QtYabause::getVDICore( int id )
 
 CDInterface QtYabause::defaultCDCore()
 {
+#ifndef UNKNOWN_ARCH
 	return ArchCD;
+#else
+	return DummyCD;
+#endif
 }
 
 SoundInterface_struct QtYabause::defaultSNDCore()
