@@ -16,7 +16,7 @@
 
 	You should have received a copy of the GNU General Public License
 	along with Yabause; if not, write to the Free Software
-	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 #ifndef UISETTINGS_H
 #define UISETTINGS_H
@@ -41,7 +41,7 @@ protected:
 
 protected slots:
 	void tbBrowse_clicked();
-	void pbInputs_clicked();
+	void on_cbInput_currentIndexChanged( int id );
 	void accept();
 };
 
