@@ -27,7 +27,7 @@
 
 SH2_struct *MSH2=NULL;
 SH2_struct *SSH2=NULL;
-static SH2_struct *CurrentSH2;
+SH2_struct *CurrentSH2;
 SH2Interface_struct *SH2Core=NULL;
 extern SH2Interface_struct *SH2CoreList[];
 
@@ -72,8 +72,14 @@ int SH2Init(int coreid)
       }
    }
 
-   if (SH2Core)
-      SH2Core->Init();
+   if (SH2Core) {
+      if (SH2Core->Init() != 0) {
+         free(MSH2);
+         free(SSH2);
+         MSH2 = SSH2 = NULL;
+         return -1;
+      }
+   }
 
    return 0;
 }
@@ -103,14 +109,14 @@ void SH2Reset(SH2_struct *context)
    
    // Reset general registers
    for (i = 0; i < 15; i++)
-      context->regs.R[i] = 0x00000000;
-                   
-   context->regs.SR.all = 0x000000F0;
-   context->regs.GBR = 0x00000000;
-   context->regs.VBR = 0x00000000;
-   context->regs.MACH = 0x00000000;
-   context->regs.MACL = 0x00000000;
-   context->regs.PR = 0x00000000;
+      SH2Core->SetGPR(context, i, 0x00000000);
+
+   SH2Core->SetSR(context, 0x000000F0);
+   SH2Core->SetGBR(context, 0x00000000);
+   SH2Core->SetVBR(context, 0x00000000);
+   SH2Core->SetMACH(context, 0x00000000);
+   SH2Core->SetMACL(context, 0x00000000);
+   SH2Core->SetPR(context, 0x00000000);
 
    // Internal variables
    context->delay = 0x00000000;
@@ -118,19 +124,19 @@ void SH2Reset(SH2_struct *context)
    context->isIdle = 0;
 
    context->frc.leftover = 0;
-   context->frc.div = 8;
+   context->frc.shift = 3;
  
    context->wdt.isenable = 0;
    context->wdt.isinterval = 1;
-   context->wdt.div = 2;
+   context->wdt.shift = 1;
    context->wdt.leftover = 0;
 
    // Reset Interrupts
    memset((void *)context->interrupts, 0, sizeof(interrupt_struct) * MAX_INTERRUPTS);
-   context->NumberOfInterrupts = 0;
+   SH2Core->SetInterrupts(context, 0, context->interrupts);
 
    // Core specific reset
-   SH2Core->Reset();
+   SH2Core->Reset(context);
 
    // Reset Onchip modules
    OnchipReset(context);
@@ -139,8 +145,9 @@ void SH2Reset(SH2_struct *context)
 //////////////////////////////////////////////////////////////////////////////
 
 void SH2PowerOn(SH2_struct *context) {
-   context->regs.PC = MappedMemoryReadLong(context->regs.VBR);
-   context->regs.R[15] = MappedMemoryReadLong(context->regs.VBR+4);
+   u32 VBR = SH2Core->GetVBR(context);
+   SH2Core->SetPC(context, MappedMemoryReadLong(VBR));
+   SH2Core->SetGPR(context, 15, MappedMemoryReadLong(VBR+4));
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -149,23 +156,6 @@ void FASTCALL SH2Exec(SH2_struct *context, u32 cycles)
 {
    CurrentSH2 = context;
 
-   // Handle interrupts
-   if (context->NumberOfInterrupts != 0)
-   {
-      if (context->interrupts[context->NumberOfInterrupts-1].level > context->regs.SR.part.I)
-      {
-         context->regs.R[15] -= 4;
-         MappedMemoryWriteLong(context->regs.R[15], context->regs.SR.all);
-         context->regs.R[15] -= 4;
-         MappedMemoryWriteLong(context->regs.R[15], context->regs.PC);
-         context->regs.SR.part.I = context->interrupts[context->NumberOfInterrupts-1].level;
-         context->regs.PC = MappedMemoryReadLong(context->regs.VBR + (context->interrupts[context->NumberOfInterrupts-1].vector << 2));
-         context->NumberOfInterrupts--;
-         context->isIdle = 0;
-         context->isSleeping = 0;
-      }
-   }
-
    SH2Core->Exec(context, cycles);
 
    FRTExec(cycles);
@@ -181,36 +171,7 @@ void FASTCALL SH2Exec(SH2_struct *context, u32 cycles)
 
 void SH2SendInterrupt(SH2_struct *context, u8 vector, u8 level)
 {
-   u32 i, i2;
-   interrupt_struct tmp;
-
-   // Make sure interrupt doesn't already exist
-   for (i = 0; i < context->NumberOfInterrupts; i++)
-   {
-      if (context->interrupts[i].vector == vector)
-         return;
-   }
-
-   context->interrupts[context->NumberOfInterrupts].level = level;
-   context->interrupts[context->NumberOfInterrupts].vector = vector;
-   context->NumberOfInterrupts++;
-
-   // Sort interrupts
-   for (i = 0; i < (context->NumberOfInterrupts-1); i++)
-   {
-      for (i2 = i+1; i2 < context->NumberOfInterrupts; i2++)
-      {
-         if (context->interrupts[i].level > context->interrupts[i2].level)
-         {
-            tmp.level = context->interrupts[i].level;
-            tmp.vector = context->interrupts[i].vector;
-            context->interrupts[i].level = context->interrupts[i2].level;
-            context->interrupts[i].vector = context->interrupts[i2].vector;
-            context->interrupts[i2].level = tmp.level;
-            context->interrupts[i2].vector = tmp.vector;
-         }
-      }
-   }
+   SH2Core->SendInterrupt(context, vector, level);
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -225,14 +186,14 @@ void SH2NMI(SH2_struct *context)
 
 void SH2Step(SH2_struct *context)
 {
-   u32 tmp = context->regs.PC;
+   u32 tmp = SH2Core->GetPC(context);
 
    // Execute 1 instruction
    SH2Exec(context, context->cycles+1);
 
    // Sometimes it doesn't always execute one instruction,
    // let's make sure it did
-   if (tmp == context->regs.PC)
+   if (tmp == SH2Core->GetPC(context))
       SH2Exec(context, context->cycles+1);
 }
 
@@ -241,7 +202,7 @@ void SH2Step(SH2_struct *context)
 void SH2GetRegisters(SH2_struct *context, sh2regs_struct * r)
 {
    if (r != NULL) {
-      memcpy(r, &context->regs, sizeof(context->regs));
+      SH2Core->GetRegisters(context, r);
    }
 }
 
@@ -250,7 +211,7 @@ void SH2GetRegisters(SH2_struct *context, sh2regs_struct * r)
 void SH2SetRegisters(SH2_struct *context, sh2regs_struct * r)
 {
    if (r != NULL) {
-      memcpy(&context->regs, r, sizeof(context->regs));
+      SH2Core->SetRegisters(context, r);
    }
 }
 
@@ -291,7 +252,7 @@ int SH2AddCodeBreakpoint(SH2_struct *context, u32 addr) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-void SH2SortCodeBreakpoints(SH2_struct *context) {
+static void SH2SortCodeBreakpoints(SH2_struct *context) {
    int i, i2;
    u32 tmp;
 
@@ -361,7 +322,7 @@ void SH2ClearCodeBreakpoints(SH2_struct *context) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-u8 FASTCALL SH2MemoryBreakpointReadByte(u32 addr) {
+static u8 FASTCALL SH2MemoryBreakpointReadByte(u32 addr) {
    int i;
 
    for (i = 0; i < CurrentSH2->bp.nummemorybreakpoints; i++)
@@ -396,7 +357,7 @@ u8 FASTCALL SH2MemoryBreakpointReadByte(u32 addr) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-u16 FASTCALL SH2MemoryBreakpointReadWord(u32 addr) {
+static u16 FASTCALL SH2MemoryBreakpointReadWord(u32 addr) {
    int i;
 
    for (i = 0; i < CurrentSH2->bp.nummemorybreakpoints; i++)
@@ -431,7 +392,7 @@ u16 FASTCALL SH2MemoryBreakpointReadWord(u32 addr) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-u32 FASTCALL SH2MemoryBreakpointReadLong(u32 addr) {
+static u32 FASTCALL SH2MemoryBreakpointReadLong(u32 addr) {
    int i;
 
    for (i = 0; i < CurrentSH2->bp.nummemorybreakpoints; i++)
@@ -466,7 +427,7 @@ u32 FASTCALL SH2MemoryBreakpointReadLong(u32 addr) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL SH2MemoryBreakpointWriteByte(u32 addr, u8 val) {
+static void FASTCALL SH2MemoryBreakpointWriteByte(u32 addr, u8 val) {
    int i;
 
    for (i = 0; i < CurrentSH2->bp.nummemorybreakpoints; i++)
@@ -506,7 +467,7 @@ void FASTCALL SH2MemoryBreakpointWriteByte(u32 addr, u8 val) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL SH2MemoryBreakpointWriteWord(u32 addr, u16 val) {
+static void FASTCALL SH2MemoryBreakpointWriteWord(u32 addr, u16 val) {
    int i;
 
    for (i = 0; i < CurrentSH2->bp.nummemorybreakpoints; i++)
@@ -546,7 +507,7 @@ void FASTCALL SH2MemoryBreakpointWriteWord(u32 addr, u16 val) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-void FASTCALL SH2MemoryBreakpointWriteLong(u32 addr, u32 val) {
+static void FASTCALL SH2MemoryBreakpointWriteLong(u32 addr, u32 val) {
    int i;
 
    for (i = 0; i < CurrentSH2->bp.nummemorybreakpoints; i++)
@@ -586,7 +547,7 @@ void FASTCALL SH2MemoryBreakpointWriteLong(u32 addr, u32 val) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-int CheckForMemoryBreakpointDupes(SH2_struct *context, u32 addr, u32 flag, int *which)
+static int CheckForMemoryBreakpointDupes(SH2_struct *context, u32 addr, u32 flag, int *which)
 {
    int i;
 
@@ -716,7 +677,7 @@ int SH2AddMemoryBreakpoint(SH2_struct *context, u32 addr, u32 flags) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-void SH2SortMemoryBreakpoints(SH2_struct *context) {
+static void SH2SortMemoryBreakpoints(SH2_struct *context) {
    int i, i2;
    memorybreakpoint_struct tmp;
 
@@ -905,24 +866,68 @@ u8 FASTCALL OnchipReadByte(u32 addr) {
 */
          return CurrentSH2->onchip.SSR;
       case 0x005:
-//         LOG("Receive Data Register read: %02X PC = %08X\n", CurrentSH2->onchip.RDR, CurrentSH2->regs.PC);
+//         LOG("Receive Data Register read: %02X PC = %08X\n", CurrentSH2->onchip.RDR, SH2Core->GetPC(CurrentSH2));
          return CurrentSH2->onchip.RDR;
+      case 0x010:
+         return CurrentSH2->onchip.TIER;
       case 0x011:
          return CurrentSH2->onchip.FTCSR;
       case 0x012:         
          return CurrentSH2->onchip.FRC.part.H;
       case 0x013:
          return CurrentSH2->onchip.FRC.part.L;
+      case 0x014:
+         if (!(CurrentSH2->onchip.TOCR & 0x10))
+            return CurrentSH2->onchip.OCRA >> 8;
+         else
+            return CurrentSH2->onchip.OCRB >> 8;
+      case 0x015:
+         if (!(CurrentSH2->onchip.TOCR & 0x10))
+            return CurrentSH2->onchip.OCRA & 0xFF;
+         else
+            return CurrentSH2->onchip.OCRB & 0xFF;
       case 0x016:
          return CurrentSH2->onchip.TCR;
       case 0x017:
          return CurrentSH2->onchip.TOCR;
+      case 0x018:
+         return CurrentSH2->onchip.FICR >> 8;
+      case 0x019:
+         return CurrentSH2->onchip.FICR & 0xFF;
+      case 0x060:
+         return CurrentSH2->onchip.IPRB >> 8;
+      case 0x062:
+         return CurrentSH2->onchip.VCRA >> 8;
+      case 0x063:
+         return CurrentSH2->onchip.VCRA & 0xFF;
+      case 0x064:
+         return CurrentSH2->onchip.VCRB >> 8;
+      case 0x065:
+         return CurrentSH2->onchip.VCRB & 0xFF;
+      case 0x066:
+         return CurrentSH2->onchip.VCRC >> 8;
+      case 0x067:
+         return CurrentSH2->onchip.VCRC & 0xFF;
+      case 0x068:
+         return CurrentSH2->onchip.VCRD >> 8;
       case 0x080:
          return CurrentSH2->onchip.WTCSR;
       case 0x081:
          return CurrentSH2->onchip.WTCNT;
       case 0x092:
          return CurrentSH2->onchip.CCR;
+      case 0x0E0:
+         return CurrentSH2->onchip.ICR >> 8;
+      case 0x0E1:
+         return CurrentSH2->onchip.ICR & 0xFF;
+      case 0x0E2:
+         return CurrentSH2->onchip.IPRA >> 8;
+      case 0x0E3:
+         return CurrentSH2->onchip.IPRA & 0xFF;
+      case 0x0E4:
+         return CurrentSH2->onchip.VCRWDT >> 8;
+      case 0x0E5:
+         return CurrentSH2->onchip.VCRWDT & 0xFF;
       default:
          LOG("Unhandled Onchip byte read %08X\n", (int)addr);
          break;
@@ -938,10 +943,34 @@ u16 FASTCALL OnchipReadWord(u32 addr) {
    {
       case 0x060:
          return CurrentSH2->onchip.IPRB;
+      case 0x062:
+         return CurrentSH2->onchip.VCRA;
+      case 0x064:
+         return CurrentSH2->onchip.VCRB;
+      case 0x066:
+         return CurrentSH2->onchip.VCRC;
+      case 0x068:
+         return CurrentSH2->onchip.VCRD;
       case 0x0E0:
          return CurrentSH2->onchip.ICR;
       case 0x0E2:
          return CurrentSH2->onchip.IPRA;
+      case 0x0E4:
+         return CurrentSH2->onchip.VCRWDT;
+      case 0x1E2: // real BCR1 register is located at 0x1E2-0x1E3; Sega Rally OK
+         return CurrentSH2->onchip.BCR1;
+      case 0x1E6:
+         return CurrentSH2->onchip.BCR2;
+      case 0x1EA:
+         return CurrentSH2->onchip.WCR;
+      case 0x1EE:
+         return CurrentSH2->onchip.MCR;
+      case 0x1F2:
+         return CurrentSH2->onchip.RTCSR;
+      case 0x1F6:
+         return CurrentSH2->onchip.RTCNT;
+      case 0x1FA:
+         return CurrentSH2->onchip.RTCOR;
       default:
          LOG("Unhandled Onchip word read %08X\n", (int)addr);
          return 0;
@@ -979,14 +1008,42 @@ u32 FASTCALL OnchipReadLong(u32 addr) {
       case 0x11C: // Acts as a separate register, but is set to the same value
       case 0x13C: // as DVDNTL after division
          return CurrentSH2->onchip.DVDNTUL;
+      case 0x180:
+         return CurrentSH2->onchip.SAR0;
+      case 0x184:
+         return CurrentSH2->onchip.DAR0;
+      case 0x188:
+         return CurrentSH2->onchip.TCR0;
       case 0x18C:
          return CurrentSH2->onchip.CHCR0;
+      case 0x190:
+         return CurrentSH2->onchip.SAR1;
+      case 0x194:
+         return CurrentSH2->onchip.DAR1;
+      case 0x198:
+         return CurrentSH2->onchip.TCR1;
       case 0x19C:
          return CurrentSH2->onchip.CHCR1;
+      case 0x1A0:
+         return CurrentSH2->onchip.VCRDMA0;
+      case 0x1A8:
+         return CurrentSH2->onchip.VCRDMA1;
       case 0x1B0:
          return CurrentSH2->onchip.DMAOR;
       case 0x1E0:
          return CurrentSH2->onchip.BCR1;
+      case 0x1E4:
+         return CurrentSH2->onchip.BCR2;
+      case 0x1E8:
+         return CurrentSH2->onchip.WCR;
+      case 0x1EC:
+         return CurrentSH2->onchip.MCR;
+      case 0x1F0:
+         return CurrentSH2->onchip.RTCSR;
+      case 0x1F4:
+         return CurrentSH2->onchip.RTCNT;
+      case 0x1F8:
+         return CurrentSH2->onchip.RTCOR;
       default:
          LOG("Unhandled Onchip long read %08X\n", (int)addr);
          return 0;
@@ -1017,7 +1074,7 @@ void FASTCALL OnchipWriteByte(u32 addr, u8 val) {
          CurrentSH2->onchip.SCR = val;
          return;
       case 0x003:
-//         LOG("Transmit Data Register write: %02X. PC = %08X\n", val, CurrentSH2->regs.PC);
+//         LOG("Transmit Data Register write: %02X. PC = %08X\n", val, SH2Core->GetPC(CurrentSH2));
          CurrentSH2->onchip.TDR = val;
          return;
       case 0x004:
@@ -1039,6 +1096,7 @@ void FASTCALL OnchipWriteByte(u32 addr, u8 val) {
          return;
       case 0x011:
          CurrentSH2->onchip.FTCSR = (CurrentSH2->onchip.FTCSR & (val & 0xFE)) | (val & 0x1);
+         return;
       case 0x012:
          CurrentSH2->onchip.FRC.part.H = val;
          return;
@@ -1063,13 +1121,13 @@ void FASTCALL OnchipWriteByte(u32 addr, u8 val) {
          switch (val & 3)
          {
             case 0:
-               CurrentSH2->frc.div = 8;
+               CurrentSH2->frc.shift = 3;
                break;
             case 1:
-               CurrentSH2->frc.div = 32;
+               CurrentSH2->frc.shift = 5;
                break;
             case 2:
-               CurrentSH2->frc.div = 128;
+               CurrentSH2->frc.shift = 7;
                break;
             case 3:
                LOG("FRT external input clock not implemented.\n");
@@ -1150,6 +1208,18 @@ void FASTCALL OnchipWriteWord(u32 addr, u16 val) {
       case 0x060:
          CurrentSH2->onchip.IPRB = val & 0xFF00;
          return;
+      case 0x062:
+         CurrentSH2->onchip.VCRA = val & 0x7F7F;
+         return;
+      case 0x064:
+         CurrentSH2->onchip.VCRB = val & 0x7F7F;
+         return;
+      case 0x066:
+         CurrentSH2->onchip.VCRC = val & 0x7F7F;
+         return;
+      case 0x068:
+         CurrentSH2->onchip.VCRD = val & 0x7F7F;
+         return;
       case 0x080:
          // This and RSTCSR have got to be the most wackiest register
          // mappings I've ever seen
@@ -1160,28 +1230,28 @@ void FASTCALL OnchipWriteWord(u32 addr, u16 val) {
             switch (val & 7)
             {
                case 0:
-                  CurrentSH2->wdt.div = 2;
+                  CurrentSH2->wdt.shift = 1;
                   break;
                case 1:
-                  CurrentSH2->wdt.div = 64;
+                  CurrentSH2->wdt.shift = 6;
                   break;
                case 2:
-                  CurrentSH2->wdt.div = 128;
+                  CurrentSH2->wdt.shift = 7;
                   break;
                case 3:
-                  CurrentSH2->wdt.div = 256;
+                  CurrentSH2->wdt.shift = 8;
                   break;
                case 4:
-                  CurrentSH2->wdt.div = 512;
+                  CurrentSH2->wdt.shift = 9;
                   break;
                case 5:
-                  CurrentSH2->wdt.div = 1024;
+                  CurrentSH2->wdt.shift = 10;
                   break;
                case 6:
-                  CurrentSH2->wdt.div = 4096;
+                  CurrentSH2->wdt.shift = 12;
                   break;
                case 7:
-                  CurrentSH2->wdt.div = 8192;
+                  CurrentSH2->wdt.shift = 13;
                   break;
             }
 
@@ -1213,6 +1283,9 @@ void FASTCALL OnchipWriteWord(u32 addr, u16 val) {
       case 0x0E2:
          CurrentSH2->onchip.IPRA = val & 0xFFF0;
          return;
+      case 0x0E4:
+         CurrentSH2->onchip.VCRWDT = val & 0x7F7F;
+         return;
       case 0x108:
       case 0x128:
          CurrentSH2->onchip.DVCR = val & 0x3;
@@ -1487,25 +1560,18 @@ void FASTCALL DataArrayWriteLong(u32 addr, u32 val)  {
 
 //////////////////////////////////////////////////////////////////////////////
 
-#ifdef PSP_TIMING_TWEAKS
 void FRTExec(u32 cycles)
-#else
-void FRTExec(UNUSED u32 cycles)
-#endif
 {
    u32 frcold;
    u32 frctemp;
+   u32 mask;
 
    frcold = frctemp = (u32)CurrentSH2->onchip.FRC.all;
+   mask = (1 << CurrentSH2->frc.shift) - 1;
    
    // Increment FRC
-#ifdef PSP_TIMING_TWEAKS
-   frctemp += ((cycles + CurrentSH2->frc.leftover) / CurrentSH2->frc.div);
-   CurrentSH2->frc.leftover = (cycles + CurrentSH2->frc.leftover) % CurrentSH2->frc.div;
-#else
-   frctemp += ((CurrentSH2->cycles + CurrentSH2->frc.leftover) / CurrentSH2->frc.div);
-   CurrentSH2->frc.leftover = (CurrentSH2->cycles + CurrentSH2->frc.leftover) % CurrentSH2->frc.div;
-#endif
+   frctemp += ((cycles + CurrentSH2->frc.leftover) >> CurrentSH2->frc.shift);
+   CurrentSH2->frc.leftover = (cycles + CurrentSH2->frc.leftover) & mask;
 
    // Check to see if there is or was a Output Compare A match
    if (frctemp >= CurrentSH2->onchip.OCRA && frcold < CurrentSH2->onchip.OCRA)
@@ -1554,13 +1620,15 @@ void FRTExec(UNUSED u32 cycles)
 
 void WDTExec(u32 cycles) {
    u32 wdttemp;
+   u32 mask;
 
    if (!CurrentSH2->wdt.isenable || CurrentSH2->onchip.WTCSR & 0x80 || CurrentSH2->onchip.RSTCSR & 0x80)
       return;
 
    wdttemp = (u32)CurrentSH2->onchip.WTCNT;
-   wdttemp += ((cycles + CurrentSH2->wdt.leftover) / CurrentSH2->wdt.div);
-   CurrentSH2->wdt.leftover = (cycles + CurrentSH2->wdt.leftover) % CurrentSH2->wdt.div;
+   mask = (1 << CurrentSH2->wdt.shift) - 1;
+   wdttemp += ((cycles + CurrentSH2->wdt.leftover) >> CurrentSH2->wdt.shift);
+   CurrentSH2->wdt.leftover = (cycles + CurrentSH2->wdt.leftover) & mask;
 
    // Are we overflowing?
    if (wdttemp > 0xFF)
@@ -1705,6 +1773,7 @@ void DMATransfer(u32 *CHCR, u32 *SAR, u32 *DAR, u32 *TCR, u32 *VCRDMA)
             *TCR = 0;
             break;
       }
+      SH2WriteNotify(destInc<0?*DAR:*DAR-i*destInc,i*abs(destInc));
    }
 
    if (*CHCR & 0x4)
@@ -1765,6 +1834,7 @@ int SH2SaveState(SH2_struct *context, FILE *fp)
 {
    int offset;
    IOCheck_struct check;
+   sh2regs_struct regs;
 
    // Write header
    if (context->isslave == 0)
@@ -1776,13 +1846,25 @@ int SH2SaveState(SH2_struct *context, FILE *fp)
    }
 
    // Write registers
-   ywrite(&check, (void *)&context->regs, sizeof(sh2regs_struct), 1, fp);
+   SH2GetRegisters(context, &regs);
+   ywrite(&check, (void *)&regs, sizeof(sh2regs_struct), 1, fp);
 
    // Write onchip registers
    ywrite(&check, (void *)&context->onchip, sizeof(Onchip_struct), 1, fp);
 
    // Write internal variables
+   // FIXME: write the clock divisor rather than the shift amount for
+   // backward compatibility (fix this next time the save state version
+   // is updated)
+   context->frc.shift = 1 << context->frc.shift;
    ywrite(&check, (void *)&context->frc, sizeof(context->frc), 1, fp);
+   {
+      u32 div = context->frc.shift;
+      context->frc.shift = 0;
+      while ((div >>= 1) != 0)
+         context->frc.shift++;
+   }
+   context->NumberOfInterrupts = SH2Core->GetInterrupts(context, context->interrupts);
    ywrite(&check, (void *)context->interrupts, sizeof(interrupt_struct), MAX_INTERRUPTS, fp);
    ywrite(&check, (void *)&context->NumberOfInterrupts, sizeof(u32), 1, fp);
    ywrite(&check, (void *)context->AddressArray, sizeof(u32), 0x100, fp);
@@ -1801,20 +1883,29 @@ int SH2SaveState(SH2_struct *context, FILE *fp)
 int SH2LoadState(SH2_struct *context, FILE *fp, UNUSED int version, int size)
 {
    IOCheck_struct check;
+   sh2regs_struct regs;
 
    if (context->isslave == 1)
       yread(&check, (void *)&yabsys.IsSSH2Running, 1, 1, fp);
 
    // Read registers
-   yread(&check, (void *)&context->regs, sizeof(sh2regs_struct), 1, fp);
+   yread(&check, (void *)&regs, sizeof(sh2regs_struct), 1, fp);
+   SH2SetRegisters(context, &regs);
 
    // Read onchip registers
    yread(&check, (void *)&context->onchip, sizeof(Onchip_struct), 1, fp);
 
    // Read internal variables
    yread(&check, (void *)&context->frc, sizeof(context->frc), 1, fp);
+   {  // FIXME: backward compatibility hack (see SH2SaveState() comment)
+      u32 div = context->frc.shift;
+      context->frc.shift = 0;
+      while ((div >>= 1) != 0)
+         context->frc.shift++;
+   }
    yread(&check, (void *)context->interrupts, sizeof(interrupt_struct), MAX_INTERRUPTS, fp);
    yread(&check, (void *)&context->NumberOfInterrupts, sizeof(u32), 1, fp);
+   SH2Core->SetInterrupts(context, context->NumberOfInterrupts, context->interrupts);
    yread(&check, (void *)context->AddressArray, sizeof(u32), 0x100, fp);
    yread(&check, (void *)context->DataArray, sizeof(u8), 0x1000, fp);
    yread(&check, (void *)&context->delay, sizeof(u32), 1, fp);
@@ -1823,6 +1914,16 @@ int SH2LoadState(SH2_struct *context, FILE *fp, UNUSED int version, int size)
    yread(&check, (void *)&context->isIdle, sizeof(u8), 1, fp);
    yread(&check, (void *)&context->instruction, sizeof(u16), 1, fp);
 
+   #if defined(SH2_DYNAREC)
+   if(SH2Core->id==2) {
+     invalidate_all_pages();
+     if (context->isslave == 1) {
+       // If the slave SH2 isn't running, make sure the dynarec stops it
+       if(!yabsys.IsSSH2Running) SH2Core->Reset(SSH2);
+     }
+   }
+   #endif
+
    return size;
 }
 
