@@ -32,13 +32,20 @@
 #include "yabause.h"
 #include "movie.h"
 
+#ifdef _arch_dreamcast
+# include "dreamcast/localtime.h"
+#endif
+#ifdef PSP
+# include "psp/localtime.h"
+#endif
+
 Smpc * SmpcRegs;
 u8 * SmpcRegsT;
 SmpcInternal * SmpcInternalVars;
 
 //////////////////////////////////////////////////////////////////////////////
 
-int SmpcInit(u8 regionid) {
+int SmpcInit(u8 regionid, int clocksync, u32 basetime) {
    if ((SmpcRegsT = (u8 *) calloc(1, sizeof(Smpc))) == NULL)
       return -1;
  
@@ -49,6 +56,8 @@ int SmpcInit(u8 regionid) {
   
    SmpcInternalVars->regionsetting = regionid;
    SmpcInternalVars->regionid = regionid;
+   SmpcInternalVars->clocksync = clocksync;
+   SmpcInternalVars->basetime = basetime ? basetime : time(NULL);
 
    return 0;
 }
@@ -114,34 +123,33 @@ void SmpcReset(void) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-void SmpcSSHON() {
+static void SmpcSSHON(void) {
    YabauseStartSlave();
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void SmpcSSHOFF() {
+static void SmpcSSHOFF(void) {
    YabauseStopSlave();
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void SmpcSNDON() {
-   M68KReset();
-   yabsys.IsM68KRunning = 1;
+static void SmpcSNDON(void) {
+   M68KStart();
    SmpcRegs->OREG[31] = 0x6;
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void SmpcSNDOFF() {
-   yabsys.IsM68KRunning = 0;
+static void SmpcSNDOFF(void) {
+   M68KStop();
    SmpcRegs->OREG[31] = 0x7;
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void SmpcCKCHG352() {
+void SmpcCKCHG352(void) {
    // Reset VDP1, VDP2, SCU, and SCSP
    Vdp1Reset();  
    Vdp2Reset();  
@@ -164,7 +172,7 @@ void SmpcCKCHG352() {
 
 //////////////////////////////////////////////////////////////////////////////
 
-void SmpcCKCHG320() {
+void SmpcCKCHG320(void) {
    // Reset VDP1, VDP2, SCU, and SCSP
    Vdp1Reset();  
    Vdp2Reset();  
@@ -202,7 +210,7 @@ int noon= 43200;
 
 //////////////////////////////////////////////////////////////////////////////
 
-void SmpcINTBACKStatus(void) {
+static void SmpcINTBACKStatus(void) {
    // return time, cartidge, zone, etc. data
    int i;
    struct tm times;
@@ -213,15 +221,17 @@ void SmpcINTBACKStatus(void) {
    //SmpcRegs->OREG[0] = 0x0 | (SmpcInternalVars->resd << 6);  // goto setclock/setlanguage screen
     
    // write time data in OREG1-7
-   tmp = time(NULL);
+   if (SmpcInternalVars->clocksync) {
+      tmp = SmpcInternalVars->basetime + ((u64)framecounter * 1001 / 60000);
+   } else {
+      tmp = time(NULL);
+   }
 #ifdef WIN32
    memcpy(&times, localtime(&tmp), sizeof(times));
-#elif !defined(_arch_dreamcast)
-   localtime_r(&tmp, &times);
-#else
-   struct tm * internal_localtime_r(const time_t * tim_p, struct tm *res);
-
+#elif defined(_arch_dreamcast) || defined(PSP)
    internal_localtime_r(&tmp, &times);
+#else
+   localtime_r(&tmp, &times);
 #endif
    year[0] = (1900 + times.tm_year) / 1000;
    year[1] = ((1900 + times.tm_year) % 1000) / 100;
@@ -303,7 +313,7 @@ void SmpcINTBACKStatus(void) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-void SmpcINTBACKPeripheral(void) {
+static void SmpcINTBACKPeripheral(void) {
   int oregoffset;
   PortData_struct *port1, *port2;
 
@@ -411,7 +421,7 @@ void SmpcINTBACKPeripheral(void) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-void SmpcINTBACK() {
+static void SmpcINTBACK(void) {
    SmpcRegs->SF = 1;
 
    if (SmpcInternalVars->intback) {
@@ -419,7 +429,10 @@ void SmpcINTBACK() {
       ScuSendSystemManager();
       return;
    }
-   if ((SmpcInternalVars->intbackIreg0 = SmpcRegs->IREG[0]) != 0) {
+
+   //we think rayman sets 0x40 so that it breaks the intback command immediately when it blocks, 
+   //rather than having to set 0x40 in response to an interrupt
+   if (SmpcInternalVars->intbackIreg0 = (SmpcRegs->IREG[0] & 1)) {
       // Return non-peripheral data
       SmpcInternalVars->firstPeri = 1;
       SmpcInternalVars->intback = (SmpcRegs->IREG[1] & 0x8) >> 3; // does the program want peripheral data too?
@@ -441,13 +454,13 @@ void SmpcINTBACK() {
 
 //////////////////////////////////////////////////////////////////////////////
 
-void SmpcINTBACKEnd() {
+void SmpcINTBACKEnd(void) {
    SmpcInternalVars->intback = 0;
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void SmpcSETSMEM() {
+static void SmpcSETSMEM(void) {
    int i;
 
    for(i = 0;i < 4;i++)
@@ -458,14 +471,14 @@ void SmpcSETSMEM() {
 
 //////////////////////////////////////////////////////////////////////////////
 
-void SmpcNMIREQ() {
+static void SmpcNMIREQ(void) {
    SH2SendInterrupt(MSH2, 0xB, 16);
    SmpcRegs->OREG[31] = 0x18;
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void SmpcResetButton() {
+void SmpcResetButton(void) {
    // If RESD isn't set, send an NMI request to the MSH2.
    if (SmpcInternalVars->resd)
       return;
@@ -475,14 +488,14 @@ void SmpcResetButton() {
 
 //////////////////////////////////////////////////////////////////////////////
 
-void SmpcRESENAB() {
+static void SmpcRESENAB(void) {
   SmpcInternalVars->resd = 0;
   SmpcRegs->OREG[31] = 0x19;
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-void SmpcRESDISA() {
+static void SmpcRESDISA(void) {
   SmpcInternalVars->resd = 1;
   SmpcRegs->OREG[31] = 0x1A;
 }
@@ -586,7 +599,7 @@ u32 FASTCALL SmpcReadLong(USED_IF_SMPC_DEBUG u32 addr) {
 
 //////////////////////////////////////////////////////////////////////////////
 
-void SmpcSetTiming(void) {
+static void SmpcSetTiming(void) {
    switch(SmpcRegs->COMREG) {
       case 0x0:
          SMPCLOG("smpc\t: MSHON not implemented\n");
@@ -734,7 +747,7 @@ int SmpcSaveState(FILE *fp)
    int offset;
    IOCheck_struct check;
 
-   offset = StateWriteHeader(fp, "SMPC", 2);
+   offset = StateWriteHeader(fp, "SMPC", 3);
 
    // Write registers
    ywrite(&check, (void *)SmpcRegs->IREG, sizeof(u8), 7, fp);
@@ -760,6 +773,7 @@ int SmpcSaveState(FILE *fp)
 int SmpcLoadState(FILE *fp, int version, int size)
 {
    IOCheck_struct check;
+   int internalsizev2 = sizeof(SmpcInternal) - 8;
 
    // Read registers
    yread(&check, (void *)SmpcRegs->IREG, sizeof(u8), 7, fp);
@@ -777,13 +791,15 @@ int SmpcLoadState(FILE *fp, int version, int size)
    {
       // This handles the problem caused by the version not being incremented
       // when SmpcInternal was changed
-      if ((size - 48) == sizeof(SmpcInternal))
-         yread(&check, (void *)SmpcInternalVars, sizeof(SmpcInternal), 1, fp);
+      if ((size - 48) == internalsizev2)
+         yread(&check, (void *)SmpcInternalVars, internalsizev2, 1, fp);
       else if ((size - 48) == 24)
          yread(&check, (void *)SmpcInternalVars, 24, 1, fp);
       else
          fseek(fp, size - 48, SEEK_CUR);
    }
+   else if (version == 2)
+      yread(&check, (void *)SmpcInternalVars, internalsizev2, 1, fp);
    else
       yread(&check, (void *)SmpcInternalVars, sizeof(SmpcInternal), 1, fp);
 
