@@ -398,7 +398,7 @@ void EnableAutoFrameSkip(void);
 void DisableAutoFrameSkip(void);
 
 void DisplayMessage(const char* str);
-int GetOSDToggle();
+int GetOSDToggle(void);
 void SetOSDToggle(int toggle);
 
 #endif
