@@ -31,12 +31,16 @@ typedef struct
    int cellw, cellh;
    int flipfunction;
    int priority;
+   int dst;
+   /* The above fields MUST NOT BE CHANGED (including inserting new fields)
+    * unless YglSprite is also updated in ygl.h */
 
+   int cellw_bits, cellh_bits;
    int mapwh;
-   int planew, planeh;
-   int pagewh;
-   int patternwh;
-   int patterndatasize;
+   int planew, planew_bits, planeh, planeh_bits;
+   int pagewh, pagewh_bits;
+   int patternwh, patternwh_bits;
+   int patterndatasize, patterndatasize_bits;
    int specialfunction;
    u32 addr, charaddr, paladdr;
    int colornumber;
@@ -71,8 +75,23 @@ typedef struct
    int isverticalscroll;
    u32 verticalscrolltbl;
    int verticalscrollinc;
+   
+   // WindowMode
+   u8  LogicWin;    // Window Logic AND OR
+   u8  bEnWin0;     // Enable Window0
+   u8  bEnWin1;     // Enable Window1
+   u8  WindowArea0; // Window Area Mode 0
+   u8  WindowArea1; // Window Area Mode 1
+   
 } vdp2draw_struct;
 
+typedef struct 
+{
+   int  WinShowLine;
+	int WinHStart;
+	int WinHEnd;
+} vdp2WindowInfo;
+
 typedef struct
 {
    float Xst;
@@ -180,7 +199,7 @@ void FASTCALL Vdp2ParameterBPlaneAddr(vdp2draw_struct *info, int i);
 float Vdp2ReadCoefficientMode0_2(vdp2rotationparameter_struct *parameter, u32 addr);
 fixed32 Vdp2ReadCoefficientMode0_2FP(vdp2rotationparameterfp_struct *parameter, u32 addr);
 
-void VideoInitGlut();
+void VideoInitGlut(void);
 
 //////////////////////////////////////////////////////////////////////////////
 
@@ -317,17 +336,17 @@ static INLINE void ReadBitmapSize(vdp2draw_struct *info, u16 bm, int mask)
 {
    switch(bm & mask)
    {
-      case 0: info->cellw = 512;
-              info->cellh = 256;
+      case 0: info->cellw = 512;  info->cellw_bits = 9;
+              info->cellh = 256;  info->cellh_bits = 8;
               break;
-      case 1: info->cellw = 512;
-              info->cellh = 512;
+      case 1: info->cellw = 512;  info->cellw_bits = 9;
+              info->cellh = 512;  info->cellh_bits = 9;
               break;
-      case 2: info->cellw = 1024;
-              info->cellh = 256;
+      case 2: info->cellw = 1024; info->cellw_bits = 10;
+              info->cellh = 256;  info->cellh_bits = 8;
               break;
-      case 3: info->cellw = 1024;
-              info->cellh = 512;
+      case 3: info->cellw = 1024; info->cellw_bits = 10;
+              info->cellh = 512;  info->cellh_bits = 9;
               break;
    }
 }
@@ -340,16 +359,19 @@ static INLINE void ReadPlaneSize(vdp2draw_struct *info, u16 reg)
    {
       case 0:
          info->planew = info->planeh = 1;
+		 info->planew_bits = info->planeh_bits = 0;
          break;
       case 1:
-         info->planew = 2;
-         info->planeh = 1;
+         info->planew = 2; info->planew_bits = 1;
+         info->planeh = 1; info->planeh_bits = 0;
          break;
       case 3:
          info->planew = info->planeh = 2;
+		 info->planew_bits = info->planeh_bits = 1;
          break;
       default: // Not sure what 0x2 does, though a few games seem to use it
          info->planew = info->planeh = 1;
+		 info->planew_bits = info->planeh_bits = 0;
          break;
    }
 }
@@ -359,17 +381,31 @@ static INLINE void ReadPlaneSize(vdp2draw_struct *info, u16 reg)
 static INLINE void ReadPatternData(vdp2draw_struct *info, u16 pnc, int chctlwh)
 {
    if(pnc & 0x8000)
+   {
       info->patterndatasize = 1;
+	  info->patterndatasize_bits = 0;
+   }
    else
+   {
       info->patterndatasize = 2;
+	  info->patterndatasize_bits = 1;
+   }
 
    if(chctlwh)
+   {
       info->patternwh = 2;
+	  info->patternwh_bits = 1;
+   }
    else
+   {
       info->patternwh = 1;
+	  info->patternwh_bits = 0;
+   }
 
-   info->pagewh = 64/info->patternwh;
+   info->pagewh = 64>>info->patternwh_bits;
+   info->pagewh_bits = 6-info->patternwh_bits;
    info->cellw = info->cellh = 8;
+   info->cellw_bits = info->cellh_bits = 3;
    info->supplementdata = pnc & 0x3FF;
    info->auxmode = (pnc & 0x4000) >> 14;
 }
