@@ -40,39 +40,67 @@
 #define IDD_VDP2VIEWER                  134
 #define IDD_PADCONFIG1                  135
 #define IDD_MOUSECONFIG                 135
+#define IDR_ACCELERATOR1                138
+#define IDD_DIALOG2                     139
+#define IDD_PROMPT                      139
 #define MENU_RECORD_MOVIE               141
 #define MENU_PLAY_MOVIE                 142
 #define MENU_STOP_MOVIE                 143
-#define IDC_WATCHLIST 900
-#define IDC_PROMPT_EDIT 901
-#define IDC_1_BYTE 902
-#define IDC_2_BYTES 903
-#define IDC_4_BYTES 904
-#define IDC_SIGNED	905
-#define IDC_HEX 906
-#define IDC_EDIT_COMPAREADDRESS 907
-#define IDC_UNSIGNED 908
-#define ID_WATCHES_UPDOWN 909
-#define RAMMENU_FILE_AUTOLOAD 910
-#define RAMMENU_FILE_SAVEWINDOW 911
-#define RAMMENU_FILE_RECENT 912
-#define RAMMENU_FILE_SAVE 913
-#define RAMMENU_FILE_SAVEAS 914
-#define RAMMENU_FILE_OPEN 915
-#define RAMMENU_FILE_APPEND 916
-#define RAMMENU_FILE_NEW 917
-#define IDC_C_WATCH_REMOVE 918
-#define IDC_C_WATCH_EDIT 919
-#define IDD_EDITWATCH 920
-#define IDC_C_WATCH 921
-#define IDC_C_WATCH_DUPLICATE 922
-#define IDC_C_WATCH_UP 923
-#define IDC_C_WATCH_DOWN 924
-#define IDC_C_ADDCHEAT 925
-#define ID_RAM_WATCH 926
-#define IDD_RAMWATCH 927
-#define IDC_SPECIFICADDRESS 928
-#define IDC_PROMPT_TEXT 929
+#define IDC_WATCHLIST                   900
+#define IDC_1_BYTE                      902
+#define IDC_2_BYTES                     903
+#define IDC_4_BYTES                     904
+#define IDC_SIGNED                      905
+#define IDC_HEX                         906
+#define IDC_UNSIGNED                    908
+#define ID_WATCHES_UPDOWN               909
+#define RAMMENU_FILE_AUTOLOAD           910
+#define RAMMENU_FILE_SAVEWINDOW         911
+#define RAMMENU_FILE_RECENT             912
+#define RAMMENU_FILE_SAVE               913
+#define RAMMENU_FILE_SAVEAS             914
+#define RAMMENU_FILE_OPEN               915
+#define RAMMENU_FILE_APPEND             916
+#define RAMMENU_FILE_NEW                917
+#define IDC_C_WATCH_REMOVE              918
+#define IDC_C_WATCH_EDIT                919
+#define IDD_EDITWATCH                   920
+#define IDC_C_WATCH                     921
+#define IDC_C_WATCH_DUPLICATE           922
+#define IDC_C_WATCH_UP                  923
+#define IDC_C_WATCH_DOWN                924
+#define IDC_C_ADDCHEAT                  925
+#define ID_RAM_WATCH                    926
+#define IDD_RAMWATCH                    927
+#define IDC_SPECIFICADDRESS             928
+#define IDC_C_AUTOSEARCH                931
+#define IDC_MISALIGN                    932
+#define IDC_C_RESET_CHANGES             933
+#define IDC_C_UNDO                      934
+#define ID_RAM_SEARCH                   935
+#define IDD_RAMSEARCH                   945
+#define IDC_DIFFERENTFROM               946
+#define IDC_DIFFERENTBY                 947
+#define IDC_MODULO                      948
+#define IDC_EDIT_DIFFBY                 949
+#define IDC_EDIT_COMPAREADDRESS         951
+#define IDC_PROMPT_TEXT                 952
+#define IDC_PROMPT_EDIT                 953
+#define IDC_EDIT_MODBY                  960
+#define IDC_PREVIOUSVALUE               961
+#define IDC_SPECIFICVALUE               962
+#define IDC_NUMBEROFCHANGES             963
+#define IDC_EDIT_COMPAREVALUE           964
+#define IDC_EDIT_COMPARECHANGES         965
+#define IDC_RAMLIST                     991
+#define IDC_C_SEARCH                    992
+#define IDC_C_RESET                     993
+#define IDC_C_ELIMINATE                 994
+#define IDC_LESSTHAN                    995
+#define IDC_MORETHAN                    996
+#define IDC_NOMORETHAN                  997
+#define IDC_NOLESSTHAN                  998
+#define IDC_EQUALTO                     999
 #define IDC_EDITTEXT1                   1001
 #define IDC_DISCTYPECB                  1001
 #define IDC_VIDEOCORECB                 1001
@@ -208,6 +236,7 @@
 #define IDC_SETTINGSTAB                 1020
 #define IDC_CPB                         1020
 #define IDC_CHEATSEARCHET               1020
+#define IDC_PROMPT_TEXT2                1020
 #define IDC_XPB                         1021
 #define IDC_CTSEARCHRESTARTBT           1021
 #define IDC_YPB                         1022
@@ -300,6 +329,8 @@
 #define IDC_SAVEMD2                     1116
 #define IDC_BUTTON5                     1117
 #define IDC_SAVEMD3                     1117
+#define ID_CANCEL                       1118
+#define IDC_EDIT1                       1119
 #define IDM_CHOOSEBIOS                  40001
 #define IDM_CHOOSECDROM                 40002
 #define IDM_MEMTRANSFER                 40004
@@ -311,8 +342,8 @@
 #define IDM_SAVESTATEAS                 40010
 #define IDM_LOADSTATEAS                 40011
 #define IDM_EXIT                        40012
-#define IDM_FILE_RECORDAVI              40015
-#define IDM_FILE_STOPAVI                40016
+#define IDM_FILE_RECORDAVI              40013
+#define IDM_FILE_STOPAVI                40014
 #define IDM_CHEATSEARCH                 40020
 #define IDM_CHEATLIST                   40021
 #define IDM_MSH2DEBUG                   40030
@@ -371,16 +402,63 @@
 #define IDM_SWEDISH                     40092
 #define IDM_SPANISH                     40093
 #define IDM_FRAMEADVANCEPAUSE           40096
-#define IDM_TOGGLEREADONLY				40098
+#define IDM_TOGGLEREADONLY              40098
+#define ID_ACCELERATOR40103             40103
+#define ID_ACCELERATOR40104             40104
+#define IDC_LABEL_HK1                   60001
+#define IDC_LABEL_HK2                   60002
+#define IDC_LABEL_HK3                   60003
+#define IDC_LABEL_HK4                   60004
+#define IDC_LABEL_HK5                   60005
+#define IDC_LABEL_HK6                   60006
+#define IDC_LABEL_HK7                   60007
+#define IDC_LABEL_HK8                   60008
+#define IDC_LABEL_HK9                   60009
+#define IDC_HOTKEY1                     60011
+#define IDC_HOTKEY2                     60012
+#define IDC_HOTKEY3                     60013
+#define IDC_HOTKEY4                     60014
+#define IDC_HOTKEY5                     60015
+#define IDC_HOTKEY6                     60016
+#define IDC_HOTKEY7                     60017
+#define IDC_HOTKEY8                     60018
+#define IDC_HOTKEY9                     60019
+#define IDC_HOTKEY10                    60020
+#define IDC_HOTKEY11                    60021
+#define IDC_HOTKEY12                    60022
+#define IDC_HOTKEY13                    60023
+#define IDC_HOTKEY14                    60024
+#define IDC_HOTKEY15                    60025
+#define IDC_HOTKEY16                    60026
+#define IDC_HOTKEY17                    60027
+#define IDC_HOTKEY18                    60028
+#define IDC_HOTKEY19                    60029
+#define IDC_HOTKEY20                    60030
+#define IDC_LABEL_HK10                  60031
+#define IDC_LABEL_HK11                  60032
+#define IDC_LABEL_HK12                  60033
+#define IDC_LABEL_HK13                  60034
+#define IDC_LABEL_HK14                  60035
+#define IDC_LABEL_HK15                  60036
+#define IDC_LABEL_HK16                  60037
+#define IDC_LABEL_HK17                  60038
+#define IDC_LABEL_HK19                  60040
+#define IDC_LABEL_HK20                  60041
+#define IDC_LABEL_HK18                  60049
+#define IDC_HKCOMBO                     60077
+#define IDD_KEYCUSTOM                   60078
+#define IDM_HOTKEY_CONFIG               60079
+#define IDC_LABEL_BLUE                  60080
+#define IDM_OPENCUEISO                  60081
 #define IDC_STATIC                      -1
 
 // Next default values for new objects
 // 
 #ifdef APSTUDIO_INVOKED
 #ifndef APSTUDIO_READONLY_SYMBOLS
-#define _APS_NEXT_RESOURCE_VALUE        138
-#define _APS_NEXT_COMMAND_VALUE         40103
-#define _APS_NEXT_CONTROL_VALUE         1118
+#define _APS_NEXT_RESOURCE_VALUE        140
+#define _APS_NEXT_COMMAND_VALUE         40106
+#define _APS_NEXT_CONTROL_VALUE         1120
 #define _APS_NEXT_SYMED_VALUE           133
 #endif
 #endif
